#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# %%
"""
Created on Tue Mar 22 10:21:27 2022

@author: ageiges
"""

import datatoolbox as dt
import matplotlib.pyplot as plt

# %%
dt.import_new_source_from_remote('PRIMAP_2019')


# %% Inventory
"""
The inventory is a pandas dataframe containing all available tables, each in a row.
The index has the tableIDs and comes with the following colums : 
['variable', 'entity', 'category', 'pathway', 'scenario', 'model', 'source', 'source_name', 'source_year', 'unit']

All coulumns can be used for the later search requests.
"""
dt.inventory().head()


# %% Seach using patterns - findp
"""
Defauls is a shell style type of search suitable to be used for categories.
"""

dt.findp(source = 'PRIMAP_2019').head()

# %%
# One star (*) is giving all results for only the following level (speparated by "|") 
res = dt.findp(variable='Emissions|CH4*',
         source = 'IPCC_SR15') # "Emissions|KYOTOGHG_AR4|Marine" will be excluded
print('Singel star results:')
print(res.variable.unique())
# 
#Two stars (**) allows for any patter for the complete rest of the string
res = dt.findp(variable='Emissions|CH4**',
         source = 'IPCC_SR15') # "Emissions|KYOTOGHG_AR4|Marine" will be included

print('Two stars results:')
print(res.variable.unique())

# %% Results object

res
res.columns
res.index

# %% Datatable
# Data table
res = dt.findp(source = 'PRIMAP_2019')
table = dt.getTable(res.index[0])
print(table)

# %%

# %%
# Meta data
table.meta


# %% meta data


--->>> benchmarks


# %%
dt.findp(source='IEA*').source.unique()

# %% adapters
# Interfaces
res = dt.findp(source='IPCC_SR15', variable=[
    'Secondary Energy|Electricity|Coal|w/o CCS',
    'Secondary Energy|Electricity'],
         pathway = ['IMA15-LiStCh|IMAGE 3.0.1',
            'CD-LINKS_NPi2020_1000|WITCH-GLOBIOM 4.4',
            'CD-LINKS_NPi2020_400|WITCH-GLOBIOM 4.4',
            'ADVANCE_2020_1.5C-2100|MESSAGE-GLOBIOM 1.0',
            'SSP1-19|AIM/CGE 2.0',
            'SSP2-19|AIM/CGE 2.0',
            'TERL_15D_LowCarbonTransportPolicy|AIM/CGE 2.1',
            'TERL_15D_NoTransportPolicy|AIM/CGE 2.1',])


print(res)
# pyam
idf =  res.as_pyam()

# xarray
xds = res.as_xarray()
# %% new way of computing a benchmark
res = dt.findp(
    source='IPCC_SR15', 
    variable=[
        'Secondary Energy|Electricity|Coal|w/o CCS',
        'Secondary Energy|Electricity'],
    pathway = [
        'IMA15-LiStCh|IMAGE 3.0.1',
        'CD-LINKS_NPi2020_1000|WITCH-GLOBIOM 4.4',
        'CD-LINKS_NPi2020_400|WITCH-GLOBIOM 4.4',
        'ADVANCE_2020_1.5C-2100|MESSAGE-GLOBIOM 1.0',
        'SSP1-19|AIM/CGE 2.0',
        'SSP2-19|AIM/CGE 2.0',
        'TERL_15D_LowCarbonTransportPolicy|AIM/CGE 2.1',
        'TERL_15D_NoTransportPolicy|AIM/CGE 2.1']
    )

xds = res.as_xarray()
xds['Coal share'] = xds['Secondary Energy|Electricity|Coal|w/o CCS'] / xds['Secondary Energy|Electricity']
xds['Coal share'] = xds['Coal share'].pint.to('percent')

plt.plot(xds['Coal share'].time, xds['Coal share'].mean(dim= 'pathway').sel(region='World'))
xds['Coal share'].median(dim= 'pathway').sel(region='World', time=2030).item()
plt.title('Coal share')
#print(xds)

# %%
dt.conversionFactor('Mt N2O / yr', 't CO2eq /yr', context ='GWPAR4')

# %% Emission intensity of power generation
res = dt.findp(
    source='IPCC_SR15', 
    variable=[
        'Emissions|CO2|Energy|Supply|Electricity',
        'Secondary Energy|Electricity'],
    pathway = ['IMA15-LiStCh|IMAGE 3.0.1',
        'CD-LINKS_NPi2020_1000|WITCH-GLOBIOM 4.4',
        'CD-LINKS_NPi2020_400|WITCH-GLOBIOM 4.4',
        'ADVANCE_2020_1.5C-2100|MESSAGE-GLOBIOM 1.0',
        'SSP1-19|AIM/CGE 2.0',
        'SSP2-19|AIM/CGE 2.0',
        'TERL_15D_LowCarbonTransportPolicy|AIM/CGE 2.1',
        'TERL_15D_NoTransportPolicy|AIM/CGE 2.1']
    )

xds = res.as_xarray()

xds['Emission intensity'] = xds['Emissions|CO2|Energy|Supply|Electricity'] / xds['Secondary Energy|Electricity']
xds['Emission intensity'] = xds['Emission intensity'].pint.to('g CO2 /  kWh')

#plt.plot(xds['Emission intensity'].time, xds['Emission intensity'].mean(dim= 'pathway').sel(region='World'))
xds['Emission intensity'].median(dim= 'pathway').sel(region='World', time=2030).item()
#plt.title(f'Emission intensity[{xds["Emission intensity"].pint.units}]')
print(xds)


# %% plotting time series
plt.figure(1)
plt.clf()

ax = plt.subplot(1,2,1)
xds['Emission intensity'].median(dim='pathway').plot.line(ax  =ax, x='time')
ax = plt.subplot(1,2,2)
xds['Emission intensity'].mean(dim= 'pathway').sel(region='World').plot.line(ax  =ax, x='time')
plt.legend(['World'])
plt.ylabel('')

# %%



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Aug 21 08:52:37 2020

@author: ageiges
"""

import datatoolbox as dt
import numpy.testing as npt
import numpy as np

from util_for_testing import df1, df_datetime


data = np.asarray([[1,2.2,3,4,5 ],
                   [1, np.nan, 4, np.nan, np.nan],
                   [1.3, np.nan, np.nan, np.nan, np.nan],
                   [np.nan, 3.4, 2.4, 3.2, np.nan]])
df3 =   df = dt.Datatable(data, 
                  columns = [2010, 2012, 2013, 2015, 2014], 
                  index = ['ARG', 'DEU', 'FRA', 'GBR'],
                  meta={'entity' : 'Emissions|CO2',
                       'scenario' : 'Historic',
                       'source' : 'XYZ_2020',
                       'unit' : 'm'}, )

def test_interpolation():
    from datatoolbox.tools.for_datatables import interpolate
    
    
    #interpolation to with columns as years
    resTable = interpolate(df1)
    
    assert (resTable.loc['DEU',2012] == 3)
    assert np.isnan(resTable.loc['GBR',2010])
    assert np.isnan(resTable.loc['DEU',2015])
    assert np.isnan(resTable.loc['FRA',2013])
    
    
    #interpolation to with columns as years
    interpolated_table = df_datetime.interpolate()
    
    assert interpolated_table.loc['GBR', '2018-08-09 12:00:00'] == 11.
    assert interpolated_table.loc['GBR', '2018-08-09 13:00:00'] == 12.
    # test of linked method
    df1.interpolate()
   
def test_aggregation():
    
    from datatoolbox.tools.for_datatables import aggregate_region
    
    mapping= {'EU3': ['DEU', 'GBR', 'FRA']}
    res, missingCountries = aggregate_region(df1, mapping, skipna=True)
    
    npt.assert_array_almost_equal(res.loc['EU3',:].values, 
                                  np.array([2.3, 3.4, 6.4, 3.2]),
                                  decimal = 6)
    npt.assert_array_almost_equal(res.loc['GBR',:].values, 
                                  np.array([np.nan, 3.4, 2.4, 3.2]),
                                  decimal = 6)
    
    
def test_growth_rates():
    
    from datatoolbox.tools.for_datatables import growth_rate
    
    res = growth_rate(df3)
    
    exp = np.array([[np.nan,  1.2 ,  0.36363636,  0.33333333,  0.25      ],
                    [np.nan, np.nan, np.nan, np.nan, np.nan],
                    [np.nan, np.nan, np.nan, np.nan, np.nan],
                    [np.nan, np.nan, -0.29411765,  0.33333333, np.nan]])
    
    npt.assert_almost_equal(res.values, exp, decimal=8)
    assert res.meta['unit'] == 'm'
    
if __name__== '__main__':
    test_interpolation()
    test_aggregation()
    test_growth_rates()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Nov 13 10:32:50 2019

@author: Andreas Geiges
"""

"""
PYMAGICC TOOLS
"""
#%%
import pymagicc as pym
import pandas as pd
import datatoolbox as dt

#%%

#%% CONFIG for MAGICC 6
REQUIRED_GASES = {'HFC143a' : 'kt',
                'FossilCO2' : 'Mt',
                'HFC125' : 'kt',
                'BC' : 'Mt',
                'NOx' : 'MtN',
                'C6F14' : 'kt',
                'SF6' : 'kt',
                'CO' : 'Mt',
                'NMVOC' : 'Mt',
                'HFC23' : 'kt',
                'HFC134a' : 'kt',
                'HFC245fa' : 'kt',
                'CH4' : 'Mt',
                'OtherCO2' : 'Mt',
                'OC' : 'Mt',
                'NH3' : 'Mt',
                'N2O' : 'MtN2O-N',
                'HFC32' : 'kt',
                'HFC43-10' : 'kt',
                'HFC227ea' : 'kt',
                'C2F6' : 'kt',
                'SOx' : 'MtS',
                'CF4' : 'kt',}
PRIMAP_PATH = '/media/sf_Documents/primap_climatemodule/'

def writeMagic6ScenFile(table, filePath):
    def _create_column_header(headers):
        return pd.MultiIndex.from_arrays(
            arrays=[ headers["variables"], headers["todos"], headers["units"], headers["regions"]],
            names=("variable", "todo", "unit", "region"))
    efCopy = table.copy()
    
    
    ##% Magicc6 requires some chemical components in specific units
    #   which are converted in the following
    
    # ensure Mt as unit for the following component converions
#    for entity in ['NH3', 'FossilCO2', 'OtherCO2' ]:
#        efCopy.changeUnit(entity, 'Mt')
#    
#    #efCopy.convert(('N2O','N2O','MtN'), 14/28)
#    efCopy.convert(('NH3','NH3','MtN'), 14/17)
#    efCopy.convert(('FossilCO2', 'CO2', 'GtC'), 12/44/1000)
#    efCopy.convert(('OtherCO2', 'CO2', 'GtC'), 12/44/1000)
    #efCopy.convert(('Sulfur','Sulfur','MtS'), 32/64)
    #efCopy.convert(('NOx','NOx','MtN'), 14/44)
    
    
    headers = dict()
    headers['variables'] = list(efCopy.columns)
    headers['variables'] = pym.io.convert_magicc6_to_magicc7_variables(list(efCopy.columns))
    
    
    headers['units'] = list(efCopy.loc['Yrs',:])
    nCols = len(headers['units'])
    headers['regions'] = ['World']*nCols
    headers['todos'] = ['']*nCols
    
    columnIndex = _create_column_header(headers)
    
    #print(variables)
    ioData = pym.io.MAGICCData()
    #return efCopy
    efCopy = efCopy.iloc[1:,:].astype(float)
    ioData.df = pd.DataFrame(efCopy)
    ioData.df.index.name = "time"
    ioData.df.columns = columnIndex
    ioData.df = pd.DataFrame(ioData.df.T.stack(), columns=["value"]).reset_index()
    ioData.write(filepath=filePath, magicc_version=6)

def run_MAGGIC6(scenFile):
    scenario = scenFile.split('.')[0]
    import os 
    os.system('matlab -nodesktop -r "run_magicc6 ' + scenario + ' ' + PRIMAP_PATH + '"')
    
    outTemperature = dt.io.read_MAGICC6_MATLAB_bulkout(scenario + '/CSSENS_DAT_SURFACE_TEMP_BOXGLOBAL_MAGICC_INDIVIDUAL.OUT')
#    outTemperature = dt.io.read_MAGICC6_MATLAB_bulkout(scenario + '/CSSENS_DAT_SURFACE_TEMP_BOXGLOBAL_MAGICC_COMPARISON_MEAN.OUT')
    print('Median: ' + str(outTemperature.loc[2100,:].median()))
    print('p66: ' + str(outTemperature.loc[2100,:].quantile(.66)))
    print('p16: ' + str(outTemperature.loc[2100,:].quantile(.16)))
    print('max Median: ' + str(outTemperature.loc[:,:].median(axis=1).max()))
    return outTemperature


#dt.greenhouse_gas_database
if __name__ == '__main__':
#    scenFile = 'WEO_2019_18.SCEN'
#    testData = pd.read_excel('/media/sf_Documents/data_files/IEA_trial_MAGICC.xlsx',index_col=0, sheet_name = 'IEA_SDS_18')
#    scenFile = 'WEO_2019_15b.SCEN'
#    testData = pd.read_excel('/media/sf_Documents/data_files/IEA_trial_MAGICC.xlsx',index_col=0, sheet_name = 'IEA_SDS_15_b')
    scenFile = 'WEO_2019_test'
    
    for scen in ['WEO_2019_test.SCEN']:#['IEA B2DS 4', 'IEA B2DS 2 rev', 'IEA B2DS 3']:
        # scenFile = scen.replace(' ','_')+ '.SCEN'
        # testData = pd.read_excel('/media/sf_Documents/data_files/Summary_of_IEA_B2DS_pathways_for_MAGICC.xlsx',index_col=0, sheet_name = scen)
        # writeMagic6ScenFile(testData, scenFile)
        outTemperature = run_MAGGIC6(scenFile)
        
        import matplotlib.pylab as plt
        
        plt.plot(outTemperature.loc[outTemperature.index[::5],:].quantile(.66,axis=1))
        plt.plot(outTemperature.loc[outTemperature.index[::5],:].quantile(.5,axis=1))
        plt.plot(outTemperature.loc[outTemperature.index[::5],:].quantile(.14,axis=1))
        
        outDf = pd.DataFrame(columns = outTemperature.index[::5])
        outDf.loc['p84',:] = outTemperature.loc[outTemperature.index[::5],:].quantile(.84,axis=1)
        outDf.loc['p75',:] = outTemperature.loc[outTemperature.index[::5],:].quantile(.75,axis=1)
        outDf.loc['p66',:] = outTemperature.loc[outTemperature.index[::5],:].quantile(.66,axis=1)
        outDf.loc['p50',:] = outTemperature.loc[outTemperature.index[::5],:].quantile(.5,axis=1)
        outDf.loc['p36',:] = outTemperature.loc[outTemperature.index[::5],:].quantile(.36,axis=1)
        outDf.loc['p25',:] = outTemperature.loc[outTemperature.index[::5],:].quantile(.25,axis=1)
        outDf.loc['p16',:] = outTemperature.loc[outTemperature.index[::5],:].quantile(.16,axis=1)
        
        scenFile = scen.replace(' ','_')
        
        outDf.to_excel('MAGICC6_RUN_' + scenFile.split('.')[0] + '.xlsx')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Sep 15 11:28:02 2020

@author: ageiges
"""


import xarray as xr
import numpy as np
import pandas as pd
#%%

def compute_percentile(data, 
                        perc, 
                        weights=None):
    """
    Fnction to compute the unweighted and weighted percentile for 
    clustered country-level data

    Parameters
    ----------
    data : TYPE
        DESCRIPTION.
    perc : TYPE
        DESCRIPTION.
    weights : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    """
    
    # un-weighted
    if weights is None:
        mask = ~np.isnan(data)
        data = data[mask]
        result  = np.percentile(data, perc*100)
        return result
    
    
    # weighted
    mask = ~np.isnan(data)
    data = data[mask]
    weights = weights[mask]
    
    
    ix = np.argsort(data)
    
    # Sort data and weights
    sort_data = data[ix]
    sort_weights = weights[ix]
    
    cdf = (np.cumsum(sort_weights) - 0.5 * sort_weights) / np.sum(sort_weights)
    result = np.interp(perc, cdf, sort_data)
    
    return result


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jun  2 13:57:46 2020

@author: ageiges
"""
import numpy as np
import datatoolbox as dt 


#%% Datatables for testing 
data = np.asarray([[1,2.2,3,4 ],
                    [1, np.nan, 4, np.nan],
                    [1.3, np.nan, np.nan, np.nan],
                    [np.nan, 3.4, 2.4, 3.2]])


df = dt.Datatable(data, 
                  columns = [2010, 2012, 2013, 2015], 
                  index = ['ARG', 'DEU', 'FRA', 'GBR'],
                  meta={'entity' : 'Emissions|CO2',
                        'scenario' : 'Historic',
                        'source' : 'XYZ_2020',
                        'unit' : 'm'}, )

sourceMeta = {'SOURCE_ID': 'XYZ_2020',
              'collected_by': dt.config.CRUNCHER,
              'date': 'last day',
              'source_url': 'www.sandbox.de',
              'licence': 'free for all'}    

metaDict2 = meta={ 'entity' : 'GDP|PPP',
                    'category' : 'Trade|imports',
                    'scenario' : 'Historic',
                    'source' : 'XYZ_2020',
                    'unit' : 'USD 2010'}
    
data2 = np.asarray([[1,22,3,4 ],
                    [23, np.nan, 34, 15],
                    [13, 1e6, np.nan, 41],
                    [np.nan, 34, 27.4, 32]])
data3 = np.asarray([[1,22,3,4 ],
                    [23, np.nan, 34, 15],
                    [13, 1e6, np.nan, 41],
                    [np.nan, 34, 27.4, 32]])

df1 = df
df2 = dt.Datatable(data2, meta=metaDict2, columns = [2008, 2012, 2013, 2015], index = ['TUN', 'DEU', 'FRA', 'GBR'])
df3 = dt.Datatable(data3, meta=metaDict2, columns = [2008, 2012, 2013, 2015], index = ['TUN', 'DEU', 'FRA', 'GBR'])
df3 =   df = dt.Datatable(data, 
                  columns = [2010, 2012, 2013, 2015], 
                  index = ['ARG', 'DEU', 'FRA', 'GBR'],
                  meta={'entity' : 'Emissions|CO2',
                        'scenario' : 'Historic',
                        'source' : 'XYZ_2020',
                        'unit' : 'm'}, )
df3.loc['ARG', 2014] = 5


# %%Datetime datatable
data_datetime = np.asarray([[1, 22    , 3     ,4  ],
                           [23, np.nan, 34    , 15],
                           [13, 1e2   , np.nan, 41],
                           [10, np.nan, np.nan, 35]])

meta_datetime = { 'entity' : 'GDP|PPP',
                   'category' : 'Trade|imports',
                   'scenario' : 'Historic',
                   'source' : 'table_with_datetime',
                   'unit' : 'USD 2010',
                   '_timeformat' : '%Y-%m-%d %H:%M:%S'}

df_datetime = dt.Datatable(data_datetime, 
                           meta=meta_datetime, 
                           columns = ['2018-08-09 11:00:00', '2018-08-09 12:00:00', '2018-08-09 13:00:00', '2018-08-10 12:00:00'], 
                           index = ['TUN', 'DEU', 'FRA', 'GBR'])
from datatoolbox.tools.for_datatables import interpolate
# print(interpolate(df_datetime).values)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar  1 14:15:56 2019

@author: andreas geiges
"""
import os
import platform
import appdirs
OS = platform.system() #'win32' , linux, #Darwin


MODULE_PATH = os.path.dirname(__file__)

# if OS == 'Darwin':
#     import matplotlib
#     matplotlib.use("TkAgg")

 
# personal configuration path
from appdirs import user_data_dir
appname = "datatoolbox"
appauthor = "ageiges"
CONFIG_DIR = user_data_dir(appname, appauthor)
#os.makedirs(CONFIG_DIR, exist_ok = True)
#%% general setup

DATATABLE_COLUMN_NAME = 'time'
DATATABLE_INDEX_NAME  = 'region'

REQUIRED_META_FIELDS = {'entity',
                        'scenario',
                        'source_name',
                        'source_year',
                        'unit',
                        'timeformat'}

META_DEFAULTS = {'_timeformat'  : '%Y'}

OPTIONAL_META_FIELDS = ['category',
                       'model']

ID_FIELDS = ['variable',
             'pathway',
             'source']

SUB_FIELDS = {'variable' : ['entity', 'category'],
              'pathway' : ['scenario', 'model'],
              'source' : ['source_name', 'source_year']}

SUB_SEP = {'variable' :'|',
          'pathway' : '|',
          'source' : '_'}

ID_SEPARATOR     = '__'


INVENTORY_FIELDS = ['variable',
                    'entity',
                    'category', 
                    'pathway',
                    'scenario',
                    'model',
                    'source',
                    'source_name',
                    'source_year',
                    'unit'
                    ]

SOURCE_META_FIELDS = ['SOURCE_ID',
                      'collected_by',
                      'date',
                      'source_url',
                      'licence',
                      'git_commit_hash']


#%% Personal setup
if not os.path.isfile(os.path.join(CONFIG_DIR, 'personal.py')):
    from .admin import create_initial_config, _create_empty_datashelf
    modulePath =  os.path.dirname(__file__) + '/'
    CRUNCHER, PATH_TO_DATASHELF, DB_READ_ONLY, DEBUG = create_initial_config(MODULE_PATH,
                                                                             CONFIG_DIR)
    _create_empty_datashelf(PATH_TO_DATASHELF, 
                           MODULE_PATH,
                           SOURCE_META_FIELDS,
                           INVENTORY_FIELDS,
                           force_new=True)
    TEST_ENV = True
else:
    # from .settings.personal import CRUNCHER, PATH_TO_DATASHELF, DB_READ_ONLY, DEBUG, AUTOLOAD_SOURCES
    
    #importng CRUNCHER, PATH_TO_DATASHELF, DB_READ_ONLY, DEBUG, AUTOLOAD_SOURCES
    try:
        config_file = os.path.join(CONFIG_DIR, 'personal.py')
        exec(open(config_file).read())
    except:
        raise(Exception(f'Error in config file: {config_file}. Please correct'))
        
    TEST_ENV = False


try:
    import xarray as xr
    AVAILABLE_XARRAY = True
except:
    
    AVAILABLE_XARRAY = False   
    if DEBUG:
        print('xarray not available: Use "pip install xarray" to install')
try:
    import docx 
    AVAILABLE_DOCX =True
except:
    AVAILABLE_DOCX = False   
    if DEBUG:
        print('xarray not available: Use "pip install python-docx" to install')
        

META_DECLARATION = '### META ###\n'
DATA_DECLARATION = '### DATA ###\n'

MODULE_PATH = os.path.dirname(__file__)
MODULE_DATA_PATH =  os.path.join(MODULE_PATH, 'data')

MAPPING_FILE_PATH = os.path.join(MODULE_DATA_PATH, 'region_mappings.csv')
CONTINENT_FILE_PATH = os.path.join(MODULE_DATA_PATH, 'all.csv')
PATH_TO_MAPPING = os.path.join(PATH_TO_DATASHELF, 'mappings')
PATH_TO_COUNTRY_FILE = os.path.join(PATH_TO_MAPPING, 'country_codes.csv')
PATH_TO_REGION_FILE =  os.path.join(PATH_TO_MAPPING, 'regions.csv')
PATH_PINT_DEFINITIONS = os.path.join(MODULE_PATH, 'pint_definitions.txt')

SOURCE_FILE   =  os.path.join(PATH_TO_DATASHELF, 'sources.csv')




SOURCE_SUB_FOLDERS = ['tables',
                      'raw_data'
                      ]

GHG_GAS_TABLE_FILE = 'GHG_properties_2019_CA.csv'
GHG_NAMING_FILENAME = 'GHG_alternative_naming.pkl'


COUNTRY_LIST = ['AFG', 'ALA', 'ALB', 'DZA', 'ASM', 'AND', 'AGO', 'AIA', 'ATA', 'ATG', 'ARG', 'ARM', 'ABW', 
                'AUS', 'AUT', 'AZE', 'BHS', 'BHR', 'BGD', 'BRB', 'BLR', 'BEL', 'BLZ', 'BEN', 'BMU', 'BTN', 
                'BOL', 'BIH', 'BES', 'BWA', 'BVT', 'BRA', 'IOT', 'BRN', 'BGR', 'BFA', 'BDI', 'KHM', 'CMR', 
                'CAN', 'CPV', 'CYM', 'CAF', 'TCD', 'CHL', 'CHN', 'CXR', 'CCK', 'COL', 'COM', 'COG', 'COD', 
                'COK', 'CRI', 'CIV', 'HRV', 'CUB', 'CUW', 'CYP', 'CZE', 'DNK', 'DJI', 'DMA', 'DOM', 'ECU', 
                'EGY', 'SLV', 'GNQ', 'ERI', 'EST', 'ETH', 'FLK', 'FRO', 'FJI', 'FIN', 'FRA', 'GUF', 'PYF', 
                'ATF', 'GAB', 'GMB', 'GEO', 'DEU', 'GHA', 'GIB', 'GRC', 'GRL', 'GRD', 'GLP', 'GUM', 'GTM', 
                'GGY', 'GIN', 'GNB', 'GUY', 'HTI', 'HMD', 'VAT', 'HND', 'HKG', 'HUN', 'ISL', 'IND', 'IDN', 
                'IRN', 'IRQ', 'IRL', 'IMN', 'ISR', 'ITA', 'JAM', 'JPN', 'JEY', 'JOR', 'KAZ', 'KEN', 'KIR', 
                'PRK', 'KOR', 'KWT', 'KGZ', 'LAO', 'LVA', 'LBN', 'LSO', 'LBR', 'LBY', 'LIE', 'LTU', 'LUX', 
                'MAC', 'MKD', 'MDG', 'MWI', 'MYS', 'MDV', 'MLI', 'MLT', 'MHL', 'MTQ', 'MRT', 'MUS', 'MYT', 
                'MEX', 'FSM', 'MDA', 'MCO', 'MNG', 'MNE', 'MSR', 'MAR', 'MOZ', 'MMR', 'NAM', 'NRU', 'NPL', 
                'NLD', 'NCL', 'NZL', 'NIC', 'NER', 'NGA', 'NIU', 'NFK', 'NOR', 'OMN', 'PAK', 'PLW', 'PSE', 
                'PAN', 'PNG', 'PRY', 'PER', 'PHL', 'PCN', 'POL', 'PRT', 'PRI', 'QAT', 'REU', 'ROU', 'RUS', 
                'RWA', 'BLM', 'SHN', 'KNA', 'LCA', 'MAF', 'SPM', 'VCT', 'WSM', 'SMR', 'STP', 'SAU', 'SEN', 
                'SRB', 'SYC', 'SLE', 'SGP', 'SXM', 'SVK', 'SVN', 'SLB', 'SOM', 'ZAF', 'SGS', 'ESP', 'LKA', 
                'SSD', 'SDN', 'SUR', 'SJM', 'SWZ', 'SWE', 'CHE', 'SYR', 'TWN', 'TJK', 'TZA', 'THA', 'TLS', 
                'TGO', 'TKL', 'TON', 'TTO', 'TUN', 'TUR', 'TKM', 'TCA', 'TUV', 'UGA', 'UKR', 'ARE', 'GBR', 
                'USA', 'UMI', 'URY', 'UZB', 'VUT', 'VEN', 'VNM', 'VGB', 'WLF', 'ESH', 'YEM', 'ZMB', 'ZWE', 
                'MNP', 'VIR']

logTables = False

DATASHELF_REMOTE  = 'git@gitlab.com:climateanalytics/datashelf/'
DATASHELF_REMOTE_HTTPS = 'https://gitlab.com/climateanalytics/datashelf/'
#### FUNCTION TESTS ########
if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed May  6 09:30:40 2020

@author: ageiges
"""
import datatoolbox as dt
import numpy as np
from pandas.testing import assert_frame_equal

def test_meta_update():
    metaDict = {'source'   : 'TEST',
            'entity'   : 'values',
            'category' : 'cat1',
            'scenario' : '#1',
            'unit'     : 'm'}
    new_meta = dt.core._update_meta(metaDict)
    
    exp = dt.config.SUB_SEP['variable'].join(metaDict[x] for x in ['entity', 'category'])
    obs = new_meta['variable']
    assert exp == obs

def test_creation():
    metaDict = {'source' : 'TEST',
                'entity' : 'values',
                'unit' : 'm'}
    metaDict2 = {'source' : 'TEST2',
                'entity' : 'area',
                'unit' : 'km'}    
        
    data = np.asarray([[1,2.2,3,4 ],
                       [2.3, np.nan, 3.4, np.nan],
                       [1.3, np.nan, np.nan, np.nan],
                       [np.nan, 3.4, 2.4, 3.2]])
    
    
    data2 = np.asarray([[1,2.2,3,4.5 ],
                   [2.3, np.nan, 3.4, np.nan],
                   [1.1, np.nan, np.nan, np.nan],
                   [np.nan, 3.3, 2.4, np.nan]])
    
    
    df = dt.Datatable(data, meta=metaDict, columns = [2010, 2012, 2013, 2015], index = ['ARG', 'DEU', 'FRA', 'GBR'])
    df2 = dt.Datatable(data2, meta=metaDict2, columns = [2009, 2012, 2013, 2015], index = ['ARG', 'DEU', 'FRA', 'GBR'])
    
    
    assert isinstance(df, dt.Datatable)
    assert isinstance(df2, dt.Datatable)
    

def test_append():
    metaDict = {'source' : 'TEST',
                'scenario' : 'scen1',
                'entity' : 'Population',
                'unit' : 'm'}
     
    data = np.asarray([[1,2.2,3,4 ],
                       [2.3, np.nan, 3.4, np.nan],
                       [1.3, np.nan, np.nan, np.nan],
                       [np.nan, 3.4, 2.4, 3.2]])
    
    metaDict2 = {'source' : 'TEST2',
                'entity' : 'area',
                'scenario' : 'scen2',
                'unit' : 'km'}  
    data2 = np.asarray([[1,2.2,3,4.5 ],
                   [2.3, np.nan, 3.4, np.nan],
                   [1.1, np.nan, np.nan, np.nan],
                   [np.nan, 3.3, 2.4, np.nan]])
    
    metaDict3 = {'source' : 'TEST2',
                'entity' : 'Population',
                'scenario' : 'scen3',
                'unit' : 'km'} 
    data3 = np.asarray([[1,2.2,3,4.5 ],
                   [2.3, np.nan, 3.4, np.nan]])
    
    df = dt.Datatable(data, meta=metaDict, columns = [2010, 2012, 2013, 2015], index = ['ARG', 'DEU', 'RUS', 'IND'])
    df2 = dt.Datatable(data2, meta=metaDict2, columns = [2009, 2012, 2013, 2015], index = ['ARG', 'DEU', 'FRA', 'GBR'])
    df3 = dt.Datatable(data3, meta=metaDict3, columns = [2009, 2012, 2013, 2015], index = ['FRA', 'GBR'])
    
    dt_merge = df.append(df3)
    obs_vlues = np.array([[    np.nan, 1.0e+00, 2.2e+00, 3.0e+00, 4.0e+00],
                          [    np.nan, 2.3e+00,     np.nan, 3.4e+00,     np.nan],
                          [    np.nan, 1.3e+00,     np.nan,     np.nan,     np.nan],
                          [    np.nan,     np.nan, 3.4e+00, 2.4e+00, 3.2e+00],
                          [1.0e+03,     np.nan, 2.2e+03, 3.0e+03, 4.5e+03],
                          [2.3e+03,     np.nan,     np.nan, 3.4e+03,     np.nan]])

    assert (dt_merge.values == obs_vlues)[~np.isnan(dt_merge.values)] .all()
    assert dt_merge.meta['scenario'] == "computed: scen1+scen3"

def test_clean():
    
    metaDict2 = {'source' : 'TEST2',
                'entity' : 'area',
                'unit' : 'km'}    
        
    
    
    data2 = np.asarray([[1,2.2,3,np.nan ],
                   [2.3, np.nan, 3.4, np.nan],
                   [1.1, np.nan, np.nan, np.nan],
                   [np.nan, 3.3, 2.4, np.nan]])
    
    
    df2 = dt.Datatable(data2, meta=metaDict2, columns = [2009, 2012, 2013, 2015], index = ['ARG', 'DEU', 'FRA', 'USSDR'])
    exp = dt.Datatable(data2, meta=metaDict2, columns = [2009, 2012, 2013, 2015], index = ['ARG', 'DEU', 'FRA', 'USSDR'])
    # exp = exp.drop('UDSSR')
    exp = exp.drop(2015, axis=1)
    df2_clean = df2.clean()
    df2_clean == df2.clean()
    assert assert_frame_equal(df2_clean, exp) is None


def test_consistent_meta():
    #%%
    df = dt.Datatable(data=np.asarray([[2.2,3.4 ],
                                       [2.3,  3.4]]
                                       ), 
                    meta={'source' : 'TEST',
                          'entity' : 'Area',
                          'category': 'Forestery',
                          'scenario' : 'Historic',
                          'unit' : 'm'}, 
                    columns = [2010, 2012], 
                    index = ['ARG', 'DEU'])
    
    df.generateTableID()
    
    assert df.meta['variable'] == 'Area|Forestery'
    assert df.meta['pathway']  == 'Historic'
    
    # check removal of empty meta
    df.meta['category'] = np.nan
    df.meta['model'] = ''
    df.meta['description'] = ''
    df.generateTableID()
    
    assert 'category' not in df.meta.keys()
    assert 'model' not in df.meta.keys()
    assert 'description' not in df.meta.keys()
    
def test_csv_write():
    
    #%%
    df = dt.Datatable(data=np.asarray([[2.2,3.4 ],
                                       [2.3,  3.4]]
                                       ), 
                    meta={'source' : 'TEST',
                          'entity' : 'Area',
                          'category': 'Forestery',
                          'scenario' : 'Historic',
                          'unit' : 'm'}, 
                    columns = [2010, 2012], 
                    index = ['ARG', 'DEU'])
    
    df.to_csv('temp.csv')
    
    fid = open('temp.csv')
    
    content = fid.readlines()
    
    fid.close()
    
    exp = ['### META ###\n', 
           'ID,Area|Forestery__Historic__TEST\n', 
           'category,Forestery\n', 
           'entity,Area\n', 
           'pathway,Historic\n', 
           'scenario,Historic\n', 
           'source,TEST\n',
           'unit,m\n', 
           'variable,Area|Forestery\n', 
           '### DATA ###\n', 
           'region,2010,2012\n', 
           'ARG,2.2,3.4\n', 
           'DEU,2.3,3.4\n']
    
    for obs_line, exp_line in zip(content, exp):
        assert obs_line == exp_line
    
    #%%'
def test_loss_less_csv_write_read():
    #%%
    df = dt.Datatable(data=np.asarray([[2.2,3.4 ],
                                       [2.3,  3.4]]
                                       ), 
                    meta={'source' : 'TEST',
                          'entity' : 'Area',
                          'category': 'Forestery',
                          'scenario' : 'Historic',
                          'unit' : 'm'}, 
                    columns = [2010, 2012], 
                    index = ['ARG', 'DEU'])
    
    df.to_csv('temp.csv')
    
    obs = dt.read_csv('temp.csv')
    
    # assert (df == obs).all()
    from pandas.testing import assert_frame_equal
    assert assert_frame_equal(df, obs) is None
    
    
    def test_loss_less_interpolate_reduce():
        from util import df, df2
        
        df = df.loc[:,df.columns.sort_values()]
        df_int = df.interpolate()
        df_obs = df_int.reduce()
        
        assert assert_frame_equal(df, df_obs) is None
        
        df2 = df2.loc[:,df2.columns.sort_values()]
        df_int = df2.interpolate()
        df_obs = df_int.reduce()
        
        assert assert_frame_equal(df2, df_obs) is None
        
    #%%
    
    
if __name__ == '__main__':    
    test_creation()
    test_append()
    test_clean()
    test_consistent_meta()
    test_csv_write()
    test_loss_less_csv_write_read()
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Aug 21 08:46:49 2020


toolbox for datatables

@author: ageiges
"""
from copy import copy
import numpy as np
#from . import config
import datatoolbox as dt
import pandas as pd

def interpolate(datatable, method="linear"):
    
    datatable = copy(datatable)
    
    
    if method == 'linear':
        from scipy import interpolate
        import numpy as np
        
        if pd.api.types.is_datetime64_any_dtype(datatable.columns.dtype):
            xData = datatable.columns.values.astype(int) // 10**9 #timestamp in seconds
        elif pd.api.types.is_integer_dtype(datatable.columns.dtype): #assuming year as int
            xData = datatable.columns.values.astype(float)
        else:
            raise(BaseException('Expecting column dtype to be integer for years or datetime'))
            
        yData = datatable.values
        for row in yData:
            idxNan = np.isnan(row)
            if (sum(~idxNan) < 2):
                continue
            interpolator = interpolate.interp1d(xData[~idxNan], row[~idxNan], kind='linear')
            col_idx = xData[idxNan].astype(int)
            col_idx = col_idx[col_idx > xData[~idxNan].min()]
            col_idx = col_idx[col_idx < xData[~idxNan].max()]
            new_idx = idxNan & (xData > xData[~idxNan].min()) & (xData < xData[~idxNan].max())
            row[new_idx] = interpolator(col_idx)
        return datatable
    else:
        raise(NotImplementedError())
        
        
def aggregate_region(table, mapping, skipna=False):
    
    table = copy(table)
    missingCountryDict = dict()
    
    for region in mapping.keys():

        
        missingCountries = set(mapping[region]) - set(table.index)
#                print('missing countries: {}'.format(missingCountries))
        missingCountryDict[region] = list(missingCountries)
        availableCountries = set(mapping[region]).intersection(table.index)
        if len(availableCountries) >0:
            table.loc[region,:] = table.loc[availableCountries,:].sum(axis=0, skipna=skipna)

    return table, missingCountryDict


def growth_rate(datatable):
    """
    Computes the growth rates for the given datatable
    """
#    tempTable = copy(datatable)
#    years = tempTable.columns
#    completeYears = list(range(years.min() - periods, years.max()))
#    for year in set(completeYears).difference(tempTable.columns):
#        tempTable.loc[:,year] = np.nan
#    tempTable =tempTable.loc[:,completeYears]
#    growth_rates = tempTable.diff(axis=1,periods=periods).iloc[:,periods+1:] / tempTable.iloc[:,periods:-1].values
#    growth_rates = growth_rates.loc[~growth_rates.isnull().all(axis=1),:]
#    growth_rates = growth_rates.loc[:,~growth_rates.isnull().all(axis=0)]
#    
#    return growth_rates
    


    
#t0 = tempTable.loc[:,years[1:]]
#t1 = tempTable.loc[:,[x-period for x in years if x-period in tempTable.columns]]
    tempTable = copy(datatable)
    growth_rates = tempTable.loc[:,tempTable.columns] * np.nan
    for i_year in range(1,len(tempTable.columns)):
        t0 = tempTable.columns[i_year-1]
        t1 = tempTable.columns[i_year]
        
        diff = tempTable.loc[:,t1].values - tempTable.loc[:,t0].values
        growth_rate = diff / tempTable.loc[:,t0]
        growth_rates.loc[:,t1] = growth_rate
    return growth_rates


def composite_dataframe(tablePrioryList):
    
    newColumns = set()
    newIndex   = set()
    for table in tablePrioryList:
        newColumns = newColumns.union(table.columns)
        newIndex = newIndex.union(table.index)
        
    newTable = dt.Datatable(data=None, columns = newColumns, index = newIndex)
    
    metaCollection = dict()
    for table in tablePrioryList:
        newTable= newTable.combine_first(table)
        
        for key in table.meta.keys():
            if key not in metaCollection.keys():
                metaCollection[key] = set()
                
            metaCollection[key] = metaCollection[key].union([table.meta[key]])
    
    for key in metaCollection.keys():
        if len(metaCollection[key]) ==1:
            newTable.meta[key] = list(metaCollection[key])[0]
        else:
            newTable.meta[key] = 'merged - meta quantity'
    return newTable
#%%


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Oct 27 12:23:20 2020

@author: ageiges
"""

import datatoolbox as dt
import numpy as np
import pandas as pd
import os


def test_import():
    import xarray
    
    
def test_to_xarray_interface():
    #%%
    inv = dt.findp(variable = ['**Ones', '**Fives'])
    
    tableSet = dt.getTables(inv.index)
    
    
    xData = tableSet.to_xarray(dimensions= ['region', 'time', 'category'])
    
    assert xData.attrs['unit'] == 'm'
    assert xData.attrs['source'] == 'Numbers_2020'
    assert xData.attrs['entity'] == 'Numbers'

    assert xData.sum() == 120
    
    dimSize, dimList = dt.core.get_dimension_extend(tableSet, dimensions= ['region', 'time', 'pathway'])
    

def test_to_xdset_interface():

    inv = dt.findp(variable = 'Numbers**',)
    
    tableSet = dt.getTables(inv.index)
    
    
    xData = tableSet.to_xset()
    
    assert len(xData.time) == 5
    assert len(xData.region) == 4
    
    assert list(xData.data_vars) == list(tableSet.keys())
    assert xData[list(xData.data_vars)[0]].attrs['unit'] == tableSet[list(tableSet.keys())[0]].meta['unit']
    
    
def test_quantified_xdataset_from_tableset():
    res = dt.findp(source = 'Numbers_2020')
    tables = dt.getTables(res.index)
    
    ds = dt.tools.xarray.key_set_to_xdataset(tables)
    
    assert ds.dims['time'] == 5
    assert ds.dims['pathway'] == 1
    assert ds.dims['region'] == 4
    assert list(ds.keys()) == ['Numbers|Ones', 'Numbers|Fives']
    assert hasattr(ds['Numbers|Ones'], 'pint') # check that dataset is quantified with unit
    
def test_quantified_xdataset_from_query_result():
    res = dt.findp(source = 'Numbers_2020')
    # tables = dt.getTables(res.index)
    
    ds = dt.tools.xarray.load_as_xdataset(res)
    
    assert ds.dims['time'] == 5
    assert ds.dims['pathway'] == 1
    assert ds.dims['region'] == 4
    assert list(ds.keys()) == ['Numbers|Ones', 'Numbers|Fives']
    assert hasattr(ds['Numbers|Ones'], 'pint') # check that dataset is quantified with unit
    
#%%    
dt.admin.switch_database_to_testing()

test_to_xarray_interface()
#if False:
    #%%
#    import xarray as xr
#    table = tableSet['Emissions|CO2|Total__Medium|Projection__SOURCE_A_2020']
##    table = table.loc[['World'],:]
#    table2 = tableSet['Emissions|CO2|Total__Historic__SOURCE_A_2020']
#    xset = xr.Dataset({'x' : table})
#    xset['y'] = table2
#    print(xset)
#    print(xset['y'])


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Apr 29 10:57:28 2020

@author: ageiges
"""



def __getAvailableFiles():
    import sys
    from . import config
    sys.path.append(config.PATH_TO_WORKFLOWS)
    #%%
    from os import listdir
    from os.path import isfile, join
    files = [f for f in listdir(config.PATH_TO_WORKFLOWS) if isfile(join(config.PATH_TO_WORKFLOWS, f))]

    return files


for file in __getAvailableFiles():
    
    if '__' in file:
        continue
    
    __new_moduleName = file.replace('.py','')
    try:
        globals()[__new_moduleName]= __import__(__new_moduleName)    
    except:
        print("failed to import workflow for {}".format(__new_moduleName))
del file




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Dec 17 09:42:44 2019

@author: ageiges
"""
from deprecated import deprecated as _deprecated
####### SETS 
import pandas as pd
from . import config

class _Regions():
    @_deprecated(version='0.4.7', reason="Please use PATHWAYS in the future")
    def __init__(self):
        # 2020
        self.CAT_COUNTRIES_2020 = set(['ARE', 'ARG', 'AUS', 'BRA', 'BTN', 'CAN', 'CHE', 'CHL', 'CHN', 'CRI',
                                   'EU28','ETH', 'GMB', 'IDN', 'IND', 'JPN', 'KAZ', 'KOR', 'MAR', 'MEX', 'NOR',
                                   'NPL', 'NZL', 'PER', 'PHL', 'RUS', 'SAU', 'SGP', 'TUR', 'UKR', 'USA',
                                   'ZAF'])
        
        self.CAT_COUNTRIES = set(['ARE', 'ARG', 'AUS', 'BRA', 'BTN', 'CAN', 'CHE', 'CHL', 'CHN', 'COL', 'CRI', 'DEU',
                                  'EU27','ETH', 'GBR', 'GMB', 'IDN', 'IND', 'IRN', 'JPN', 'KAZ', 'KEN', 'KOR', 'MAR', 
                                  'MEX', 'NGA', 'NOR', 'NPL', 'NZL', 'PER', 'PHL', 'RUS', 'SAU', 'SGP', 'TUR', 'THA', 
                                  'UKR', 'USA', 'VNM', 'ZAF'])
        
        self.AnnexI = set(['AUS', 'AUT', 'BLR', 'BEL', 'BGR', 'CAN', 'HRV', 'CYP', 
                       'CZE', 'DNK', 'EST', 'FIN', 'FRA', 'DEU', 'GRC', 'HUN', 
                       'ISL', 'IRL', 'ITA', 'JPN', 'LVA', 'LIE', 'LTU', 'LUX', 
                       'MLT', 'MCO', 'NLD', 'NZL', 'NOR', 'POL', 'PRT', 'ROU', 
                       'RUS', 'SVK', 'SVN', 'ESP', 'SWE', 'CHE', 'TUR', 'UKR', 
                       'GBR', 'USA'])
        
        self.G20 = set(['ARG', 'AUS', 'AUT', 'BEL', 'BRA', 'BGR', 'CAN', 'CHN', 'HRV', 'CYP',
                   'CZE', 'DNK', 'EST', 'FIN', 'FRA', 'DEU', 'GRC', 'HUN', 'IND', 'IDN',
                   'IRL', 'ITA', 'JPN', 'KOR', 'LVA', 'LIE', 'LTU', 'LUX', 'MLT', 'MEX',
                   'NLD', 'POL', 'PRT', 'ROU', 'RUS', 'SAU', 'SVK', 'SVN', 'ZAF', 'ESP',
                   'SWE', 'TUR', 'GBR', 'USA'])
        
        self.EU28 = set(['AUT', 'BEL', 'BGR', 'CYP', 'CZE', 'DEU', 'DNK', 'ESP', 'EST', 'FIN',
                   'FRA', 'GBR', 'GRC', 'HRV', 'HUN', 'IRL', 'ITA', 'LIE', 'LTU', 'LUX',
                   'LVA', 'MLT', 'NLD', 'POL', 'PRT', 'ROU', 'SVK', 'SVN', 'SWE'])
        
        self.EU27 = set(['AUT', 'BEL', 'BGR', 'CYP', 'CZE', 'DEU', 'DNK', 'ESP', 'EST', 'FIN',
                   'FRA', 'GRC', 'HRV', 'HUN', 'IRL', 'ITA', 'LIE', 'LTU', 'LUX',
                   'LVA', 'MLT', 'NLD', 'POL', 'PRT', 'ROU', 'SVK', 'SVN', 'SWE'])


    
        self.LDCS = set(["AGO", "BEN", "BFA", "BDI", "CAF", "TCD", "COM", "COD", "DJI", "ERI",
                  "ETH", "GMB", "GIN", "GNB", "LSO", "LBR", "MDG", "MWI", "MLI", "MRT",
                  "MOZ", "NER", "RWA", "STP", "SEN", "SLE", "SOM", "SSD", "SDN", "TZA",
                  "TGO", "UGA", "ZMB",
                  # Asia
                  "AFG", "BGD", "BTN", "KHM", "TLS", "LAO", "MMR", "NPL", "YEM",
                  # Oceania
                  "KIR", "SLB", "TUV", "VUT",
                  # The Americas
                  "HTI"])
    
        self.SIDS = set(['GUY', 'BHS', 'BLZ', 'BRB', 'ATG', 'HTI', 'JAM', 'TTO', 'SUR', 
                     'LCA', 'VCT', 'GRD', 'DMA', 'KNA', 'MSR', 'KIR', 'MHL', 'TUV',
                     'NRU', 'NIU', 'NCL', 'SLB', 'WSM', 'GUM', 'PLW', 'FJI', 'PYF',
                     'FSM', 'ASM', 'COK', 'PNG', 'VUT', 'TON', 'MNP'])
    
        self.AR5 = set(['R5ASIA', 'R5LAM', 'R5MAF', 'R5OECD90+EU', 'R5REF', 'World'])
class _Scenarios():
    @_deprecated(version='0.4.7', reason="Please use PATHWAYS in the future")
    def __init__(self):
        
        #%% 1.5 compatible
        dict_ = dict()
        dict_['models'] = ['AIM_CGE_2.0',
                                         'AIM_CGE_2.0',
                                         'AIM_CGE_2.1',
                                         'AIM_CGE_2.1',
                                         'IMAGE_3.0.1',
                                         'IMAGE_3.0.1',
                                         'MESSAGE-GLOBIOM_1.0',
                                         'MESSAGE-GLOBIOM_1.0',
                                         'MESSAGE-GLOBIOM_1.0',
                                         'POLES_EMF33',
                                         'POLES_EMF33',
                                         'POLES_EMF33',
                                         'POLES_EMF33',
                                         'POLES_EMF33',
                                         'POLES_EMF33',
                                         'POLES_EMF33',
                                         'WITCH-GLOBIOM_4.4',
                                         'WITCH-GLOBIOM_4.4',
                                         'MESSAGEix-GLOBIOM_1.0']

        dict_['scenarios'] = ['SSP1-19',
                              'SSP2-19',
                            'TERL_15D_LowCarbonTransportPolicy',
                            'TERL_15D_NoTransportPolicy',
                            'IMA15-LiStCh',
                            'SSP1-19',
                            'ADVANCE_2020_1.5C-2100',
                            'SSP1-19',
                            'SSP2-19',
                            'EMF33_1.5C_cost100',
                            'EMF33_1.5C_limbio',
                            'EMF33_1.5C_nofuel',
                            'EMF33_WB2C_limbio',
                            'EMF33_WB2C_nobeccs',
                            'EMF33_WB2C_nofuel',
                            'EMF33_WB2C_none',
                            'CD-LINKS_NPi2020_1000',
                            'CD-LINKS_NPi2020_400',
                            'LowEnergyDemand']
        
        self.compatible_15_sustainable = pd.DataFrame(dict_)

        #%% 2.0 compatible
        dict_ = dict()
        
        dict_['models'] = ['AIM_CGE_2.0', 
                           'AIM_CGE_2.0', 
                           'AIM_CGE_2.0', 
                           'AIM_CGE_2.0', 
                           'AIM_CGE_2.0', 
                           'AIM_CGE_2.0',
                           'AIM_CGE_2.1', 
                           'AIM_CGE_2.1', 
                           'AIM_CGE_2.1', 
                           'AIM_CGE_2.1', 
                           'AIM_CGE_2.1', 
                           'IMAGE_3.0.1', 
                           'IMAGE_3.0.1', 
                           'IMAGE_3.0.1', 
                           'IMAGE_3.0.1', 
                           'IMAGE_3.0.1', 
                           'IMAGE_3.0.1', 
                           'IMAGE_3.0.1', 
                           'MESSAGE_V.3', 
                           'MESSAGE_V.3', 
                           'MESSAGE_V.3', 
                           'MESSAGE-GLOBIOM_1.0',
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGEix-GLOBIOM_1.0', 
                           'MESSAGEix-GLOBIOM_1.0', 
                           'POLES_ADVANCE', 
                           'POLES_ADVANCE', 
                           'POLES_EMF33', 
                           'POLES_EMF33', 
                           'POLES_EMF33', 
                           'POLES_EMF33', 
                           'POLES_EMF33', 
                           'REMIND_1.7', 
                           'REMIND-MAgPIE_1.7-3.0',
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'WITCH-GLOBIOM_4.2', 
                           'WITCH-GLOBIOM_4.2', 
                           'WITCH-GLOBIOM_4.2', 
                           'WITCH-GLOBIOM_4.4']
        dict_['scenarios'] = ['ADVANCE_2020_WB2C', 
                              'ADVANCE_2030_Price1.5C', 
                              'ADVANCE_2030_WB2C', 
                              'SSP1-26', 
                              'SSP2-26', 
                              'SSP4-26', 
                              'CD-LINKS_NPi2020_1000', 
                              'EMF33_WB2C_cost100', 
                              'EMF33_WB2C_full', 
                              'TERL_2D_LowCarbonTransportPolicy', 
                              'TERL_2D_NoTransportPolicy', 
                              'ADVANCE_2020_WB2C', 
                              'ADVANCE_2030_WB2C', 
                              'CD-LINKS_NPi2020_1000', 
                              'IMA15-LoNCO2',
                              'SSP1-26', 
                              'SSP2-26', 
                              'SSP4-26', 
                              'GEA_Eff_1p5C', 
                              'GEA_Eff_1p5C_Delay2020', 
                              'GEA_Eff_AdvNCO2_1p5C',
                              'ADVANCE_2020_WB2C', 
                              'ADVANCE_2030_Price1.5C',
                              'ADVANCE_2030_WB2C', 
                              'EMF33_Med2C_nobeccs', 
                              'EMF33_Med2C_none', 
                              'EMF33_WB2C_cost100', 
                              'EMF33_WB2C_full', 
                              'EMF33_WB2C_limbio', 
                              'EMF33_WB2C_nofuel', 
                              'EMF33_tax_hi_full', 
                              'CD-LINKS_NPi2020_1000', 
                              'CD-LINKS_NPi2020_400',
                              'ADVANCE_2020_Med2C', 
                              'ADVANCE_2030_Med2C', 
                              'EMF33_Med2C_cost100', 
                              'EMF33_Med2C_limbio', 
                              'EMF33_Med2C_nobeccs', 
                              'EMF33_Med2C_nofuel', 
                              'EMF33_Med2C_none', 
                              'ADVANCE_2020_WB2C', 
                              'CD-LINKS_NPi2020_1000', 
                              'EMF33_WB2C_nobeccs', 
                              'EMF33_WB2C_none', 
                              'PEP_2C_full_eff', 
                              'PEP_2C_full_netzero', 
                              'PEP_2C_red_NDC', 
                              'PEP_2C_red_goodpractice', 
                              'PEP_2C_red_netzero', 
                              'SMP_2C_Def', 
                              'SMP_2C_early',
                              'ADVANCE_2020_WB2C', 
                              'ADVANCE_2030_Price1.5C', 
                              'ADVANCE_2030_WB2C', 
                              'CD-LINKS_NPi2020_1600']
        
        self.compatible_20_sustainable = pd.DataFrame(dict_)
        
        
        dict_ = {'models': ['AIM_CGE_2.0',
              'AIM_CGE_2.0',
              'AIM_CGE_2.0',
              'AIM_CGE_2.1',
              'AIM_CGE_2.1',
              'AIM_CGE_2.1',
              'C-ROADS-5.005',
              'C-ROADS-5.005',
              'C-ROADS-5.005',
              'GCAM_4.2',
              'IMAGE_3.0.1',
              'IMAGE_3.0.1',
              'IMAGE_3.0.1',
              'IMAGE_3.0.1',
              'IMAGE_3.0.1',
              'IMAGE_3.0.1',
              'IMAGE_3.0.1',
              'MERGE-ETL_6.0',
              'MESSAGE-GLOBIOM_1.0',
              'MESSAGE-GLOBIOM_1.0',
              'MESSAGE-GLOBIOM_1.0',
              'MESSAGE-GLOBIOM_1.0',
              'MESSAGE-GLOBIOM_1.0',
              'MESSAGEix-GLOBIOM_1.0',
              'POLES_ADVANCE',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'REMIND_1.5',
              'REMIND_1.5',
              'REMIND_1.5',
              'REMIND_1.5',
              'REMIND_1.7',
              'REMIND_1.7',
              'REMIND-MAgPIE_1.7-3.0',
              'REMIND-MAgPIE_1.7-3.0',
              'REMIND-MAgPIE_1.7-3.0',
              'REMIND-MAgPIE_1.7-3.0',
              'REMIND-MAgPIE_1.7-3.0',
              'REMIND-MAgPIE_1.7-3.0',
              'REMIND-MAgPIE_1.7-3.0',
              'WITCH-GLOBIOM_3.1',
              'WITCH-GLOBIOM_3.1',
              'WITCH-GLOBIOM_4.2',
              'WITCH-GLOBIOM_4.4',
              'WITCH-GLOBIOM_4.4'],
        'scenarios': ['ADVANCE_2020_1.5C-2100',
              'SSP1-19',
              'SSP2-19',
              'CD-LINKS_NPi2020_400',
              'TERL_15D_LowCarbonTransportPolicy',
              'TERL_15D_NoTransportPolicy',
              'Ratchet-1.5-limCDR-noOS',
              'Ratchet-1.5-noCDR',
              'Ratchet-1.5-noCDR-noOS',
              'SSP1-19',
              'IMA15-AGInt',
              'IMA15-Def',
              'IMA15-Eff',
              'IMA15-LiStCh',
              'IMA15-Pop',
              'IMA15-TOT',
              'SSP1-19',
              'DAC15_50',
              'ADVANCE_2020_1.5C-2100',
              'EMF33_1.5C_cost100',
              'EMF33_1.5C_full',
              'SSP1-19',
              'SSP2-19',
              'LowEnergyDemand',
              'ADVANCE_2020_1.5C-2100',
              'EMF33_1.5C_cost100',
              'EMF33_1.5C_full',
              'EMF33_1.5C_limbio',
              'EMF33_1.5C_nofuel',
              'EMF33_WB2C_cost100',
              'EMF33_WB2C_full',
              'EMF33_WB2C_limbio',
              'EMF33_WB2C_nobeccs',
              'EMF33_WB2C_nofuel',
              'EMF33_WB2C_none',
              'EMC_Def_100$',
              'EMC_LimSW_100$',
              'EMC_NucPO_100$',
              'EMC_lowEI_100$',
              'CEMICS-1.5-CDR12',
              'CEMICS-1.5-CDR8',
              'PEP_1p5C_red_eff',
              'SMP_1p5C_Def',
              'SMP_1p5C_Sust',
              'SMP_1p5C_early',
              'SMP_1p5C_lifesty',
              'SMP_1p5C_regul',
              'SMP_2C_Sust',
              'SSP1-19',
              'SSP4-19',
              'ADVANCE_2020_1.5C-2100',
              'CD-LINKS_NPi2020_1000',
              'CD-LINKS_NPi2020_400']}

        self.compatible_15_unfiltered = pd.DataFrame(dict_)
#        df = pd.read_excel(config.MODULE_DATA_PATH + '/compatible1_5_unfiltered.xlsx')
#        df.loc[:,'ref'] = df.loc[:,['Scenario','Model']].apply(lambda x: '|'.join(x.str.replace(' ','_').str.replace('/','_')),axis=1)
#        dict_ = dict()
#            
#        dict_['models'] = list(df.loc[:,'Model'].apply(lambda x: x.replace(' ','_').replace('/','_')))
#        dict_['scenarios'] = list(df.loc[:,'Scenario'].apply(lambda x: x.replace(' ','_').replace('/','_')))
#        self.compatible_15_unfiltered = pd.DataFrame(dict_)

class _Pathways():
    def __init__(self):
        
        #%% 1.5 compatible
        
        self.compatible_15_sustainable = ['SSP2-19|AIM_CGE_2.0',
                                         'TERL_15D_LowCarbonTransportPolicy|AIM_CGE_2.1',
                                         'Ratchet-1.5-noCDR|C-ROADS-5.005',
                                         'Ratchet-1.5-noCDR-noOS|C-ROADS-5.005',
                                         'IMA15-LiStCh|IMAGE_3.0.1',
                                         'SSP1-19|IMAGE_3.0.1',
                                         'SSP1-19|MESSAGE-GLOBIOM_1.0',
                                         'SSP2-19|MESSAGE-GLOBIOM_1.0',
                                         'LowEnergyDemand|MESSAGEix-GLOBIOM_1.0',
                                         'EMF33_1.5C_cost100|POLES_EMF33',
                                         'EMF33_1.5C_full|POLES_EMF33',
                                         'EMF33_1.5C_limbio|POLES_EMF33',
                                         'EMF33_1.5C_nofuel|POLES_EMF33',
                                         'EMF33_WB2C_cost100|POLES_EMF33',
                                         'EMF33_WB2C_limbio|POLES_EMF33',
                                         'EMF33_WB2C_nobeccs|POLES_EMF33',
                                         'EMF33_WB2C_nofuel|POLES_EMF33',
                                         'EMF33_WB2C_none|POLES_EMF33',
                                         'CEMICS-1.5-CDR8|REMIND_1.7',
                                         'CD-LINKS_NPi2020_400|WITCH-GLOBIOM_4.4']
        # self.compatible_15_sustainable = pd.DataFrame(dict_)

        #%% 2.0 compatible
        dict_ = dict()
        
        dict_['models'] = ['AIM_CGE_2.0', 
                           'AIM_CGE_2.0', 
                           'AIM_CGE_2.0', 
                           'AIM_CGE_2.0', 
                           'AIM_CGE_2.0', 
                           'AIM_CGE_2.0',
                           'AIM_CGE_2.1', 
                           'AIM_CGE_2.1', 
                           'AIM_CGE_2.1', 
                           'AIM_CGE_2.1', 
                           'AIM_CGE_2.1', 
                           'IMAGE_3.0.1', 
                           'IMAGE_3.0.1', 
                           'IMAGE_3.0.1', 
                           'IMAGE_3.0.1', 
                           'IMAGE_3.0.1', 
                           'IMAGE_3.0.1', 
                           'IMAGE_3.0.1', 
                           'MESSAGE_V.3', 
                           'MESSAGE_V.3', 
                           'MESSAGE_V.3', 
                           'MESSAGE-GLOBIOM_1.0',
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGE-GLOBIOM_1.0', 
                           'MESSAGEix-GLOBIOM_1.0', 
                           'MESSAGEix-GLOBIOM_1.0', 
                           'POLES_ADVANCE', 
                           'POLES_ADVANCE', 
                           'POLES_EMF33', 
                           'POLES_EMF33', 
                           'POLES_EMF33', 
                           'POLES_EMF33', 
                           'POLES_EMF33', 
                           'REMIND_1.7', 
                           'REMIND-MAgPIE_1.7-3.0',
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'REMIND-MAgPIE_1.7-3.0', 
                           'WITCH-GLOBIOM_4.2', 
                           'WITCH-GLOBIOM_4.2', 
                           'WITCH-GLOBIOM_4.2', 
                           'WITCH-GLOBIOM_4.4']
        dict_['scenarios'] = ['ADVANCE_2020_WB2C', 
                              'ADVANCE_2030_Price1.5C', 
                              'ADVANCE_2030_WB2C', 
                              'SSP1-26', 
                              'SSP2-26', 
                              'SSP4-26', 
                              'CD-LINKS_NPi2020_1000', 
                              'EMF33_WB2C_cost100', 
                              'EMF33_WB2C_full', 
                              'TERL_2D_LowCarbonTransportPolicy', 
                              'TERL_2D_NoTransportPolicy', 
                              'ADVANCE_2020_WB2C', 
                              'ADVANCE_2030_WB2C', 
                              'CD-LINKS_NPi2020_1000', 
                              'IMA15-LoNCO2',
                              'SSP1-26', 
                              'SSP2-26', 
                              'SSP4-26', 
                              'GEA_Eff_1p5C', 
                              'GEA_Eff_1p5C_Delay2020', 
                              'GEA_Eff_AdvNCO2_1p5C',
                              'ADVANCE_2020_WB2C', 
                              'ADVANCE_2030_Price1.5C',
                              'ADVANCE_2030_WB2C', 
                              'EMF33_Med2C_nobeccs', 
                              'EMF33_Med2C_none', 
                              'EMF33_WB2C_cost100', 
                              'EMF33_WB2C_full', 
                              'EMF33_WB2C_limbio', 
                              'EMF33_WB2C_nofuel', 
                              'EMF33_tax_hi_full', 
                              'CD-LINKS_NPi2020_1000', 
                              'CD-LINKS_NPi2020_400',
                              'ADVANCE_2020_Med2C', 
                              'ADVANCE_2030_Med2C', 
                              'EMF33_Med2C_cost100', 
                              'EMF33_Med2C_limbio', 
                              'EMF33_Med2C_nobeccs', 
                              'EMF33_Med2C_nofuel', 
                              'EMF33_Med2C_none', 
                              'ADVANCE_2020_WB2C', 
                              'CD-LINKS_NPi2020_1000', 
                              'EMF33_WB2C_nobeccs', 
                              'EMF33_WB2C_none', 
                              'PEP_2C_full_eff', 
                              'PEP_2C_full_netzero', 
                              'PEP_2C_red_NDC', 
                              'PEP_2C_red_goodpractice', 
                              'PEP_2C_red_netzero', 
                              'SMP_2C_Def', 
                              'SMP_2C_early',
                              'ADVANCE_2020_WB2C', 
                              'ADVANCE_2030_Price1.5C', 
                              'ADVANCE_2030_WB2C', 
                              'CD-LINKS_NPi2020_1600']
        pathways = list()
        for scenario, model in zip(dict_['scenarios'], dict_['models']):
            pathway = '|'.join([scenario, model])
            pathways.append(pathway)
        self.compatible_20_sustainable = pathways
        # self.compatible_20_sustainable = pd.DataFrame(dict_)
        
        
        dict_ = {'models': ['AIM_CGE_2.0',
              'AIM_CGE_2.0',
              'AIM_CGE_2.0',
              'AIM_CGE_2.1',
              'AIM_CGE_2.1',
              'AIM_CGE_2.1',
              'C-ROADS-5.005',
              'C-ROADS-5.005',
              'C-ROADS-5.005',
              'GCAM_4.2',
              'IMAGE_3.0.1',
              'IMAGE_3.0.1',
              'IMAGE_3.0.1',
              'IMAGE_3.0.1',
              'IMAGE_3.0.1',
              'IMAGE_3.0.1',
              'IMAGE_3.0.1',
              'MERGE-ETL_6.0',
              'MESSAGE-GLOBIOM_1.0',
              'MESSAGE-GLOBIOM_1.0',
              'MESSAGE-GLOBIOM_1.0',
              'MESSAGE-GLOBIOM_1.0',
              'MESSAGE-GLOBIOM_1.0',
              'MESSAGEix-GLOBIOM_1.0',
              'POLES_ADVANCE',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'POLES_EMF33',
              'REMIND_1.5',
              'REMIND_1.5',
              'REMIND_1.5',
              'REMIND_1.5',
              'REMIND_1.7',
              'REMIND_1.7',
              'REMIND-MAgPIE_1.7-3.0',
              'REMIND-MAgPIE_1.7-3.0',
              'REMIND-MAgPIE_1.7-3.0',
              'REMIND-MAgPIE_1.7-3.0',
              'REMIND-MAgPIE_1.7-3.0',
              'REMIND-MAgPIE_1.7-3.0',
              'REMIND-MAgPIE_1.7-3.0',
              'WITCH-GLOBIOM_3.1',
              'WITCH-GLOBIOM_3.1',
              'WITCH-GLOBIOM_4.2',
              'WITCH-GLOBIOM_4.4',
              'WITCH-GLOBIOM_4.4'],
        'scenarios': ['ADVANCE_2020_1.5C-2100',
              'SSP1-19',
              'SSP2-19',
              'CD-LINKS_NPi2020_400',
              'TERL_15D_LowCarbonTransportPolicy',
              'TERL_15D_NoTransportPolicy',
              'Ratchet-1.5-limCDR-noOS',
              'Ratchet-1.5-noCDR',
              'Ratchet-1.5-noCDR-noOS',
              'SSP1-19',
              'IMA15-AGInt',
              'IMA15-Def',
              'IMA15-Eff',
              'IMA15-LiStCh',
              'IMA15-Pop',
              'IMA15-TOT',
              'SSP1-19',
              'DAC15_50',
              'ADVANCE_2020_1.5C-2100',
              'EMF33_1.5C_cost100',
              'EMF33_1.5C_full',
              'SSP1-19',
              'SSP2-19',
              'LowEnergyDemand',
              'ADVANCE_2020_1.5C-2100',
              'EMF33_1.5C_cost100',
              'EMF33_1.5C_full',
              'EMF33_1.5C_limbio',
              'EMF33_1.5C_nofuel',
              'EMF33_WB2C_cost100',
              'EMF33_WB2C_full',
              'EMF33_WB2C_limbio',
              'EMF33_WB2C_nobeccs',
              'EMF33_WB2C_nofuel',
              'EMF33_WB2C_none',
              'EMC_Def_100$',
              'EMC_LimSW_100$',
              'EMC_NucPO_100$',
              'EMC_lowEI_100$',
              'CEMICS-1.5-CDR12',
              'CEMICS-1.5-CDR8',
              'PEP_1p5C_red_eff',
              'SMP_1p5C_Def',
              'SMP_1p5C_Sust',
              'SMP_1p5C_early',
              'SMP_1p5C_lifesty',
              'SMP_1p5C_regul',
              'SMP_2C_Sust',
              'SSP1-19',
              'SSP4-19',
              'ADVANCE_2020_1.5C-2100',
              'CD-LINKS_NPi2020_1000',
              'CD-LINKS_NPi2020_400']}

        pathways = list()
        for scenario, model in zip(dict_['scenarios'], dict_['models']):
            pathway = '|'.join([scenario, model])
            pathways.append(pathway)
        self.compatible_15_unfiltered = pathways
#%% INSTANCES
SCENARIOS = _Scenarios()
REGIONS   = _Regions()
PATHWAYS   = _Pathways()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Dec 22 11:02:57 2020

@author: ageiges
"""

#%% Datatoolbox 

def xls_default_excel_reader_setup():
    """ 
    Retrun Template for reader setup dict
    """
    print("""
        setup = dict()
        setup['filePath']  = 'path/to/template/'
        setup['fileName']  = 'template.xlsx'
        setup['sheetName'] = 'templateSheet0'
        setup['timeIdxList']  = ('B1', 'C5')
        setup['spaceIdxList'] = ('A2', 'A20')
            """) 
    

#%% Matplotlib

def plt_fancy_text_box():
    print("""
        plt.text(x=.6,y= 0.4
                 , s='text', color='k', 
        bbox={'facecolor':'white', 'alpha':0.8, 'pad':2})
        """)

def plt_legend():
    print("""
        plt.legend(handles,['first line only'], bbox_to_anchor=(.5,0), loc="lower center", bbox_transform=fig.transFigure, ncol=3, title = 'GHG-Emissions ')
        ax.legend(reversed(handles), reversed(labels), bbox_to_anchor=(1.2, .5), loc='center right', ncol=1)
        """)
    
#%% OS
def os_list_of_subfolders():
    
    print("""
        **get subfolder from folder**
        folderList = [ name for name in os.listdir(mainFolder) if os.path.isdir(os.path.join(mainFolder, name)) ]
        """)
    
#%% Pandas
    
def pd_combine_columns():
    print("""
          df['combined'] = df[['Col1', 'Col2']].apply(lambda x: '_'.join(x.str.strip()), axis=1)
          """)
    
def pd_rempove_letters():
    print("""
          df['Columm_name'].apply(lambda x: re.sub("[^0123456789\.]", "", str(x))) 
          """)


import datatoolbox as dt

import numpy.testing as npt

def test_base_conversion1():
    #testing compound units and addtional units (CO2eq)
    hr_in_yr = dt.conversionFactor('yr', 'h')
    
    try:
        npt.assert_almost_equal(hr_in_yr, 8765.812770749999) # were counting leap years now?
    except:
        npt.assert_almost_equal(hr_in_yr, 8766.0) # allow for rounding (for some openscm versions)
    obs = dt.conversionFactor('Mt CO2eq / yr', 'kg CO2eq / hr')
    exp = 1e9 / hr_in_yr
    npt.assert_almost_equal(obs, exp)

def test_custom_base_conversion1():    
    obs = dt.conversionFactor('t oil_equivalent/capita', 'MJ/capita')
    exp = 41868
    npt.assert_almost_equal(obs, exp)


def test_GWP_conversion_N2O():    
    obs = dt.conversionFactor("Mt N2O", "Gg CO2eq", context="GWPAR4")
    exp = 298000
    npt.assert_almost_equal(obs, exp)

def test_GWP_conversion_CH4():        
    obs = dt.conversionFactor("Mt CH4", "Mt CO2eq", context="GWPAR4")
    exp = 25
    npt.assert_almost_equal(obs, exp)
    
def test_HFC_units():    
    dt.core.ur('HFC134aeq') 
    
def test_function_getUnit():
    
    dt.core.getUnit('°C')
    dt.core.getUnit('$')
    dt.core.getUnit('€')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 29 10:16:36 2019

@author: andreas Geiges
"""

import os 
import re
#import pycountry
import numpy as np
import pandas as pd
from . import config as conf
#from hdx.location.country import Country
#from datatools import core


special_regions = list()   

if not os.path.exists(os.path.join(conf.PATH_TO_DATASHELF, 'mappings')):
    from .admin import create_empty_datashelf
    create_empty_datashelf(conf.PATH_TO_DATASHELF)



class RegionMapping:
    
    def __init__(self, mappingFolder = None):
        
        self.countries = [x for x in conf.COUNTRY_LIST]
        self.contextList   = list()
        self.validIDs  = list()
        #self.validIDs.extend(self.countries)
        
        if mappingFolder is None:
            self.grouping = pd.DataFrame([], index = self.countries)
            print('creating empty grouping table')
            
        else:
            fileList = os.listdir(mappingFolder)
            for file in fileList:
                if file.endswith(".csv"):
                    self.loadContext(mappingFolder, file)
                        
            self.grouping = pd.read_csv(conf.PATH_TO_REGION_FILE, index_col=0)
            for contextCol in self.grouping.columns:
                self.createNewContext(contextCol, self.grouping[contextCol])
            
#    
#
#    def exists(self, regionString):
#        for context in self.contextList:
#            if (self.grouping[context] == (regionString)).any():
#                return True, context
#        #nothing found    
#        return False, None
#
#    def allExist(self, iterable):
#        testSet = set(iterable)           
#        
#        for context in self.contextList:
#            if testSet.issubset(set(self.grouping[context])):
#                return True, context                   
#        #nothing found    
#        return False, None            

    def exists(self, regionString):
        return regionString in self.validIDs
    
    def allExist(self, iterable):
        testSet = set(iterable)
        return testSet.issubset(set(self.validIDs))
    
    def loadContext(self, folderPath, fileName):
        name = fileName.replace('mapping_','').replace('.csv','')
        mappingDataFrame =  pd.read_csv(os.path.join(folderPath, fileName), index_col=0)
        context = RegionContext(name, mappingDataFrame)
        
        # assure no regionID is duplicated
#        for regionID in context.listAll():
#            if regionID in self.valid_IDs:
#                raise(Exception(regionID + 'found as duplicate'))
        #add new regionIDs to valid IDs
        self.validIDs.extend(context.listAll())
        
        self.__setattr__(name, context)
        self.contextList.append(context)
        
    def createNewContext(self, name, mappingDict):
        context = RegionContext.fromDict(name, mappingDict, self.countries)
        self.validIDs.extend(context.listAll())
        self.__setattr__(name, context)
        self.contextList.append(context)

    def addRegionToContext(self, name, mappingDict):
        context = self.__getattribute__(name)
        context.addRegions(mappingDict)
        
        
    def save(self):
        for context in self.contextList:
            
            context.writeToCSV(conf.PATH_TO_MAPPING)    
            
    def listAll(self):
        return self.validIDs            

class RegionContext():
    
    def __init__(self, name, mappingDataFrame):
        self.name = name
        self.groupingDf = mappingDataFrame 
        
        self.keys = self.listAll
        self.__getitem__ = self.membersOf
    
    def __call__(self):
        print('Regions:')
        print()
        
    def regionOf(self, countryID):
        """ 
        Returns the membership in this context
        """
        return self.groupingDf.columns[self.groupingDf.loc[countryID]]
    
    def membersOf(self, regionName):
        return self.groupingDf.index[self.groupingDf[regionName]]
    
    def listAll(self):
        return self.groupingDf.columns
    
    def writeToCSV(self, folderPath):
        self.groupingDf.to_csv(os.path.join(folderPath, 'mapping_' + self.name + '.csv'))

    def addRegions(self, mappingDict):
        for spatialID in mappingDict.keys():
            idList = mappingDict[spatialID]
            idList = [id for id in idList if id in conf.COUNTRY_LIST]
            self.groupingDf.loc[:,spatialID] = False
            self.groupingDf.loc[idList,spatialID] = True
        
    @classmethod        
    def fromDict(cls, name, mappingDict, countries):
        if np.nan in mappingDict.keys():
            del mappingDict[np.nan]
        mappingDataFrame = pd.DataFrame(index=countries)
        
        for spatialID in mappingDict.keys():
            idList = mappingDict[spatialID]
            idList = [id for id in idList if id in countries]
            mappingDataFrame.loc[:,spatialID] = False
            mappingDataFrame.loc[idList,spatialID] = True
            
        return cls(name, mappingDataFrame)       

    def toDict(self):
        mappDict = dict()
        
        for key in self.listAll():
            mappDict[key] = list(self.membersOf(key))
        
        return mappDict
        
class CountryMapping():
    
    def  __init__(self, dataTable = None):
        
        self.countries = [x for x in conf.COUNTRY_LIST]
        self.contextList = list()
        self.nameColumns = list()
        self.numericColumns = list()
        
        if dataTable is None:
            self.codes = pd.DataFrame([], index = self.countries)
            print('creating empty grouping table')
            
        else:
            self.codes = pd.read_csv(conf.PATH_TO_COUNTRY_FILE, index_col=0)
            for contextCol in self.codes.columns:
                self.createNewContext(contextCol, self.codes[contextCol])
                if self.codes[contextCol].dtype == 'object':
                    self.nameColumns.append(contextCol)
                elif self.codes[contextCol].dtype == 'float64':
                    self.numericColumns.append(contextCol)
                
    def createNewContext(self, name, codeSeries):
        self.codes[name] = codeSeries
        self.__setattr__(name, CountryContext(name,self.codes))
        self.contextList.append(name)
        
    def save(self):
        self.codes.to_csv(conf.PATH_TO_COUNTRY_FILE)


                
#    def exists(self, regionString):
#        
#        if (self.codes.index == (regionString)).any():
#            return True, 'alpha3'
#            
#        for context in self.contextList:
#            print(regionString)
#            if (self.codes[context] == (regionString)).any():
#                return True, context
#        #nothing found
#        return False, None
    
    def exists(self, regionString):
        return regionString in self.countries
    
    def allExist(self, iterable):
        testSet = set(iterable)
        return testSet.issubset(set(self.countries))
              
    def listAll(self):
        return self.countries

class CountryContext():
    
    def __init__(self, name, codeSeries):
        self.name = name
        self.codesFromISO = codeSeries[name].reset_index().set_index(name)
        

    def __call__(self, country=None):
        if country is None:
            print(self.codesFromISO)
        else:
            return self.coCode(country)
    
    def coCode(self, country):
        """ 
        Returns the membership in this context
        """
        return self.codesFromISO.loc[country, 'index']
        
regions = RegionMapping(conf.PATH_TO_MAPPING)
countries = CountryMapping(conf.PATH_TO_MAPPING)
#groupings = Groupings()
        
            
def initializeCountriesFromData():
    from hdx.location.country import Country
    

     
    mappingDf = pd.read_csv(conf.MAPPING_FILE_PATH, index_col=0)   
    continentDf = pd.read_csv(conf.CONTINENT_FILE_PATH, index_col=0)
    continentDf = continentDf.set_index('alpha-3')
    
    countryNames = list()
    countryCodes = list()
    for code in mappingDf.index:
#        country = pycountry.countries.get(alpha_3=code)
        country = Country.get_country_info_from_iso3(code)
        if country is not None:
#            countryNames.append(country.name)
#            countryCodes.append(code)
            countryNames.append(country['#country+name+preferred'])
            countryCodes.append(code)
            
    mappingDf['name'] = pd.Series(data=countryNames, index=countryCodes)

    countryCodes = CountryMapping()
    countryCodes.createNewContext('alpha2', continentDf['alpha-2'])
    countryCodes.createNewContext('numISO', continentDf['country-code'])
    countryCodes.createNewContext('name', mappingDf['name'])
    countryCodes.createNewContext('IEA_Name', mappingDf['IEA_country'])
    return countryCodes


def mappingSeries2Dict(mappingSeries):
    outDict = dict()
    for spatialID in mappingSeries.unique():
        outDict[spatialID] = list(mappingSeries.index[(mappingSeries==spatialID)])
        
    return outDict
    
def initializeRegionsFromData():
    #%% SETUP

    if  not os.path.exists(conf.PATH_TO_MAPPING):
        os.makedirs(conf.PATH_TO_MAPPING)
    
 
    mappingDf = pd.read_csv(conf.MAPPING_FILE_PATH, index_col=0)
    continentDf = pd.read_csv(conf.CONTINENT_FILE_PATH, index_col=0)
    continentDf = continentDf.set_index('alpha-3')    
    #CONTINENTS
    
    regions = RegionMapping()
    
    continentDict = mappingSeries2Dict(continentDf['region'])
    continentDict['World'] = list(continentDf.index)
    regions.createNewContext('continent', continentDict)
    regions.createNewContext('AR5', mappingSeries2Dict(mappingDf['ar5']))
    regions.createNewContext('IEA', mappingSeries2Dict(mappingDf['IEA_region_short']))
    regions.createNewContext('IEA_long', mappingSeries2Dict(mappingDf['IEA_region_long']))
    regions.createNewContext('IAM_MESSAGE', mappingSeries2Dict(mappingDf['MESSAGE']))
    regions.createNewContext('EU28', mappingSeries2Dict(mappingDf['eu28']))
    return regions




def getMembersOfRegion(context, regionID):
    
    if context not in regions.contextList:
        raise Exception('Context ' + context + ' not defined')

    if regionID not in regions.__getattribute__(context).listAll():
        raise Exception('RegionID ' + regionID + ' not in context')            
            
    return regions.__getattribute__(context).membersOf(regionID)

def getValidSpatialIDs():
    return regions.validIDs + countries.countries + special_regions
    
def getSpatialID(descriptor, iso_type = 'alpha3'):
    """
    returns the spatial ID (ISO3) for a given desciptor that 
    can be string or in
    """
    if isinstance(descriptor, str) and not descriptor.isdigit():
        #string search
        for codeCol in countries.nameColumns:
            mask = countries.codes[codeCol]==descriptor
            if np.sum(mask) == 1:
                if iso_type =='alpha3':
                    return countries.codes.index[mask][0]
                elif iso_type  in ['alpha2', 'numISO'] :
                    return countries.codes.loc[mask,iso_type][0]
    else:
        #numeric search
        for codeCol in countries.numericColumns:
            mask = countries.codes[codeCol]==descriptor
            if np.sum(mask) == 1:
                if iso_type =='alpha3':
                    return countries.codes.index[mask][0]
                elif iso_type  in ['alpha2', 'numISO'] :
                    return countries.codes.loc[mask,iso_type][0]

def nameOfCountry(coISO):
    try:
        return countries.codes.loc[coISO,'name']
    except:
        return coISO

def add_new_special_regions(regionList):
    """
    This function allows to add special regions to the  list of valid regions 
    to allows for special exceptions if needed. 
    
    If needed, adding this new regions must be done after each new import of 
    datatoolbox since the changes are not permanent.
    """
    for region in regionList:
        if region not in special_regions:
            special_regions.append(region)
            
            
class IPCC_SR15(object):
    def region_mapping(self, 
                        org = 'native', 
                        dest='standard'):
        mapping = {
            'World' : 'World',
            'R5ROWO' : 'R5_ROWO',
            'R5ASIA' : 'R5_ASIA',
            'R5LAM' : 'R5_LAM',
            'R5MAF' : 'R5_MAF',
            'R5OECD90+EU' : 'R5_OECD',
            'R5REF' : 'R5_REF'
            }
        mapping = pd.DataFrame(mapping.items(), columns = ['native', 'standard'])
        return mapping.set_index(org).to_dict()[dest]
  
class ADVANCE(object):
    def region_mapping(self, 
                        org = 'native', 
                        dest='standard'):
        mapping = {'ASIA' :'R5_ASIA', 
                   'BRA' : 'BRA',
                   'CHN' :'CHN',
                   'EU28' : 'EU28', 
                   'IND' : 'IND',
                   'JPN' : 'JPN',
                   'LAM' : 'R5_LAM',
                   'MAF' : 'R5_MAF',
                   'OECD90+EU' : 'R5_OECD',
                   'REF' : 'R5_REF', 
                   'RUS' : 'RUS', 
                   'USA' : 'USA', 
                   'World' :  'World'}
        mapping = pd.DataFrame(mapping.items(), columns = ['native', 'standard'])
        return mapping.set_index(org).to_dict()[dest]
        
class ENGAGE(object):
    def region_mapping(self, 
                        org = 'native', 
                        dest='standard'):
        mapping = {          
            'China (R10)': 'R10CHINA+',
            'European Union and Western Europe (R10)' : 'R10EUROPE',
            'Latin America (R10)' : 'R10LATIN_AM',
            'Middle East (R10)' : 'R10MIDDLE_EAST',
            'North America (R10)' : 'R10NORTH_AM',
            'Other (R10)' : 'R10ROWO',
            'Other (R5)' :'R5_OTHER',
            'Other Asian countries (R10)' : 'R10REST_ASIA',
            'Pacific OECD (R10)' : 'R10PAC_OECD',
            'Reforming Economies (R10)': 'R10REF_ECON',
            'South Asia (R10)' : 'R10INDIA+',
            'Sub-Saharan Africa (R10)' : 'R10AFRICA',

            'Asia (R5)':  'R5ASIA',
            'Reforming Economies (R5)' : 'R5REF',  
            'Middle East & Africa (R5)' : 'R5MAF',          
            'Latin America (R5)' : 'R5LAM',
            'OECD & EU (R5)' : 'R5OECD',
            
            'World' : 'World'}
        mapping = pd.DataFrame(mapping.items(), columns = ['native', 'standard'])
        return mapping.set_index(org).to_dict()[dest]

class NGFS(object):
     def region_mapping(self, 
                        org = 'native', 
                        dest='standard'):
         mapping= {
             'World' : 'World',
             'OECD90 and EU (and EU candidate) countries' : 'R5_OECD',
             'Countries from the Reforming Ecomonies of the Former Soviet Union' : 'R5_REF',
             'Asian countries except Japan' : 'R5_ASIA',
             'Countries of the Middle East and Africa' : 'R5_MID',
             'Latin American countries' : 'R5_LAM'}
    
         mapping = pd.DataFrame(mapping.items(), columns = ['native', 'standard'])
         return mapping.set_index(org).to_dict()[dest]
    

class CD_LINKS(object):
    
    def region_mapping(self, 
                        org = 'native', 
                        dest='standard'):
        mapping= {
             'World' : 'World',
             'OECD90 and EU (and EU candidate) countries' : 'R5_OECD',
             'Countries from Reforming Ecomonies of the Former Soviet Union' : 'R5_REF',
             'Asian countries except Japan' : 'R5_ASIA',
             'Countries of the Middle East and Africa' : 'R5_MAF',
             'Latin American countries' : 'R5_LAM',
             'Federative Republic of Brazil' : 'BRA',
             "People's Repulic of China" : 'CHN',
             'European Union (28 member countries)': 'EU28',
             'Republic of India' : 'IDN', 
             'State of Japan' : 'JPN',
             'Russian Federation' :'RUS', 
             'United States of America': 'USA'}
        
        mapping = pd.DataFrame(mapping.items(), columns = ['native', 'standard'])
        return mapping.set_index(org).to_dict()[dest]

class IPCC_AR6(object):
    
    def region_mapping(self, 
                        org = 'native', 
                        dest='standard'):
        
        mapping = {'World': 'World',
         'Sao Tome and Principe': 'STP',
         'Turkmenistan': 'TKM',
         'Saint Helena': 'SHN',
         'Lithuania': 'LTU',
         'Cambodia': 'KHM',
         'Ethiopia': 'ETH',
         'Aruba': 'ABW',
         'Swaziland': None,
         'Palestine': None,
         'Bolivia': 'BOL',
         'Bahamas, The': None,
         'Burkina Faso': 'BFA',
         'Ghana': 'GHA',
         'Cape Verde': None,
         'Slovenia': 'SVN',
         'Guatemala': 'GTM',
         'Bosnia and Herzegovina': 'BIH',
         'Guinea': 'GIN',
         'Jordan': 'JOR',
         'Congo, Rep.': None,
         'Spain': 'ESP',
         'Liberia': 'LBR',
         'Netherlands': 'NLD',
         'Jamaica': 'JAM',
         'Oman': 'OMN',
         'Tanzania': None,
         'Albania': 'ALB',
         'Gabon': 'GAB',
         'Niue': 'NIU',
         'New Zealand': 'NZL',
         'Yemen': 'YEM',
         'Pakistan': 'PAK',
         'Greenland': 'GRL',
         'Samoa': 'WSM',
         'Slovak Republic': None,
         'United Arab Emirates': 'ARE',
         'Guam': 'GUM',
         'Kosovo': None,
         'Azerbaijan': 'AZE',
         'Madagascar': 'MDG',
         'Lesotho': 'LSO',
         'Saint Vincent and the Grenadines': None,
         'Kenya': 'KEN',
         'South Korea': None,
         'Belarus': 'BLR',
         'Tajikistan': 'TJK',
         'Afghanistan': 'AFG',
         'Bangladesh': 'BGD',
         'Mauritania': 'MRT',
         'Solomon Islands': 'SLB',
         'Turks and Caicos Islands': 'TCA',
         'Saint Lucia': 'LCA',
         'Cyprus': 'CYP',
         'French Polynesia': 'PYF',
         'France': 'FRA',
         'Bermuda': 'BMU',
         'Namibia': 'NAM',
         'Somalia': 'SOM',
         'Peru': 'PER',
         'Laos': None,
         'Nauru': 'NRU',
         'Seychelles': 'SYC',
         'Norway': 'NOR',
         "Cote d'Ivoire": None,
         'Cook Islands': 'COK',
         'Benin': 'BEN',
         'Western Sahara': 'ESH',
         'Cuba': 'CUB',
         'Cameroon': 'CMR',
         'Montenegro': 'MNE',
         'Saint Kitts and Nevis': 'KNA',
         'Togo': 'TGO',
         'Armenia': 'ARM',
         'Antigua and Barbuda': 'ATG',
         'Dominican Republic': 'DOM',
         'Ukraine': 'UKR',
         'Bahrain': 'BHR',
         'Tonga': 'TON',
         'Finland': 'FIN',
         'Libya': 'LBY',
         'Central African Republic': 'CAF',
         'Sweden': 'SWE',
         'British Virgin Islands': 'VGB',
         'Mali': 'MLI',
         'Bulgaria': 'BGR',
         'Mauritius': 'MUS',
         'Romania': 'ROU',
         'Angola': 'AGO',
         'Portugal': 'PRT',
         'South Africa': 'ZAF',
         'Fiji': 'FJI',
         'Brunei Darussalam': 'BRN',
         'Malaysia': 'MYS',
         'Austria': 'AUT',
         'Mozambique': 'MOZ',
         'Uganda': 'UGA',
         'Kyrgyz Republic': None,
         'Hungary': 'HUN',
         'Niger': 'NER',
         'Falkland Islands': None,
         'Kuwait': 'KWT',
         'Panama': 'PAN',
         'Guyana': 'GUY',
         'Costa Rica': 'CRI',
         'Luxembourg': 'LUX',
         'American Samoa': 'ASM',
         'Gibraltar': 'GIB',
         'Ireland': 'IRL',
         'Nigeria': 'NGA',
         'Ecuador': 'ECU',
         'Czech Republic': 'CZE',
         'Iran': None,
         'Algeria': 'DZA',
         'El Salvador': 'SLV',
         'Saint Pierre and Miquelon': 'SPM',
         'Chile': 'CHL',
         'Puerto Rico': 'PRI',
         'Belgium': 'BEL',
         'Haiti': 'HTI',
         'Iraq': 'IRQ',
         'Sierra Leone': 'SLE',
         'Georgia': 'GEO',
         'Hong Kong (China)': None,
         'Denmark': 'DNK',
         'Poland': 'POL',
         'Moldova': None,
         'Morocco': 'MAR',
         'Croatia': 'HRV',
         'Mongolia': 'MNG',
         'Guinea-Bissau': 'GNB',
         'Kiribati': 'KIR',
         'Switzerland': 'CHE',
         'Grenada': 'GRD',
         'Belize': 'BLZ',
         'Chad': 'TCD',
         'Estonia': 'EST',
         'Uruguay': 'URY',
         'Equatorial Guinea': 'GNQ',
         'Lebanon': 'LBN',
         'Uzbekistan': 'UZB',
         'Tunisia': 'TUN',
         'Djibouti': 'DJI',
         'Rwanda': 'RWA',
         'Timor-Leste': 'TLS',
         'Dominica': 'DMA',
         'Colombia': 'COL',
         'Reunion': None,
         'Burundi': 'BDI',
         'Taiwan': None,
         'Nicaragua': 'NIC',
         'Barbados': 'BRB',
         'Qatar': 'QAT',
         'Congo, Dem. Rep.': None,
         'Italy': 'ITA',
         'Bhutan': 'BTN',
         'Sudan': 'SDN',
         'Nepal': 'NPL',
         'Malta': 'MLT',
         'Maldives': 'MDV',
         'Suriname': 'SUR',
         'Cayman Islands': 'CYM',
         'Venezuela': 'VEN',
         'Israel': 'ISR',
         'Iceland': 'ISL',
         'Zambia': 'ZMB',
         'Senegal': 'SEN',
         'Papua New Guinea': 'PNG',
         'Malawi': 'MWI',
         'Macau (China)': None,
         'Trinidad and Tobago': 'TTO',
         'Zimbabwe': 'ZWE',
         'Germany': 'DEU',
         'Vanuatu': 'VUT',
         'Martinique': 'MTQ',
         'Kazakhstan': 'KAZ',
         'Philippines': 'PHL',
         'Eritrea': 'ERI',
         'Montserrat': 'MSR',
         'New Caledonia': 'NCL',
         'Macedonia': None,
         'North Korea': None,
         'Paraguay': 'PRY',
         'Latvia': 'LVA',
         'Syria': None,
         'Honduras': 'HND',
         'Myanmar': 'MMR',
         'Egypt': 'EGY',
         'Singapore': 'SGP',
         'Serbia': 'SRB',
         'Botswana': 'BWA',
         'United Kingdom': 'GBR',
         'Antarctica': 'ATA',
         'Gambia, The': None,
         'Greece': 'GRC',
         'Sri Lanka': 'LKA',
         'French Guiana': 'GUF',
         'Comoros': 'COM',
         'Micronesia (Federated States of)': 'FSM',
         'Guadeloupe': 'GLP',
         'Mayotte': 'MYT',
         'South Sudan': 'SSD',
         'Virgin Islands (U.S.)': None,
         'Curaçao': 'CUW',
         'OECD90 and EU (and EU candidate) countries (R6)': None,
         'Countries from the Reforming Ecomonies of the Former Soviet Union (R6)': None,
         'Asian countries except Japan (R6)': None,
         'Countries of the Middle East (R6)': None,
         'Countries of Sub-Saharan Africa (R6)': None,
         'Latin American countries (R6)': None,
         'Rest of the World (R6)': None,
         'OECD90 and EU (and EU candidate) countries': 'R5OECD',
         'Countries from the Reforming Ecomonies of the Former Soviet Union': 'R5REF',
         'Asian countries except Japan': 'R5ASIA',
         'Countries of the Middle East and Africa': 'R5MAF',
         'Latin American countries': 'R5LAM',
         'Rest of the World (R5)': 'R5ROWO',
         'North America; primarily the United States of America and Canada': None,
         'Eastern and Western Europe (i.e., the EU28)': None,
         'Pacific OECD': None,
         'Reforming Economies of Eastern Europe and the Former Soviet Union; primarily Russia': None,
         'Countries of centrally-planned Asia; primarily China': None,
         'Countries of South Asia; primarily India': None,
         'Other countries of Asia': None,
         'Countries of Sub-Saharan Africa': None,
         'Countries of the Middle East; Iran, Iraq, Israel, Saudi Arabia, Qatar, etc.': None,
         'Countries of Latin America and the Caribbean': None,
         'Rest of the World (R10)': None,
         'Argentina': 'ARG',
         'Australia': 'AUS',
         'Brazil': 'BRA',
         'Canada': 'CAN',
         'China': 'CHN',
         'European Union (28 member countries)': 'EU28',
         'India': 'IND',
         'Indonesia': 'IDN',
         'Japan': 'JPN',
         'Mexico': 'MEX',
         'Russia': 'RUS',
         'Saudi Arabia': 'SAU',
         'Thailand': 'THA',
         'Turkey': 'TUR',
         'United States of America': 'USA',
         'Vietnam\xa0': None,
        'R5ASIA' : 'R5ASIA', 
        'R5LAM' : 'R5LAM', 
        'R5MAF' : 'R5MAF', 
        'R5OECD90+EU' : 'R5OECD', 
        'R5REF' : 'R5REF',
        'R5ROWO' : 'R5ROWO',
        'R10ROWO' : 'R10ROWO',
        'R10AFRICA': 'R10AFRICA',
         'R10CHINA+': 'R10CHINA+',
         'R10EUROPE': 'R10EUROPE',
         'R10INDIA+': 'R10INDIA+',
         'R10LATIN_AM': 'R10LATIN_AM',
         'R10MIDDLE_EAST': 'R10MIDDLE_EAST',
         'R10NORTH_AM': 'R10NORTH_AM',
         'R10PAC_OECD': 'R10PAC_OECD',
         'R10REF_ECON': 'R10REF_ECON',
         'R10REST_ASIA': 'R10REST_ASIA',
         'R10ROWO': 'R10ROWO'}
        
        
        mapping = pd.DataFrame(mapping.items(), columns = ['native', 'standard'])
        return mapping.set_index(org).to_dict()[dest]

class IPCC_AR5(object):
    
    def region_mapping(self, 
                        org = 'native', 
                        dest='standard'):
        
        mapping = {'ASIA': 'R5_ASIA',
                   'LAM': 'R5_LAM',
                   'MAF': 'R5_MAF',
                   'OECD90': 'R5_OECD90',
                   'REF': 'R5_REF',
                   'World': 'World'}
        mapping = pd.DataFrame(mapping.items(), columns = ['native', 'standard'])
        return mapping.set_index(org).to_dict()[dest]
    
    def model_mapping(self,
                      org = 'native' , 
                      dest='PIK'):
        mapping = {
            'native' : [
                'GCAM 2.0', 'GCAM 3.0', 'GCAM 3.1', 'IMAGE 2.4', 'IMAGE 2.4 EMF22', 
                'MERGE-ETL_2011', 'MERGE_AME', 'MERGE_EMF22', 'MERGE_EMF27', 'MESSAGE V.1', 
                'MESSAGE V.2', 'MESSAGE V.3', 'MESSAGE V.4', 'REMIND 1.1', 'REMIND 1.2', 
                'REMIND 1.3', 'REMIND 1.4', 'REMIND 1.5', 'WITCH_AME', 'WITCH_AMPERE', 
                'WITCH_EMF22', 'WITCH_EMF27', 'WITCH_LIMITS', 'WITCH_ROSE', 'WITCH_RECIPE'],
            'PIK' : [
                'GC20', 'GC30', 'GC31', 'IMG24', 'IMG24E22', 
                'MRGET11', 'MRGAME', 'MRGE22', 'MRGE27', 'MESV1', 
                'MESV2', 'MESV3', 'MESV4', 'REM11', 'REM12', 
                'REM13', 'REM14', 'REM15', 'WITAME', 'WITAMP',  
                'WITE22', 'WITE27', 'WITLIM', 'WITRSE', 'WITREC']
            }
        mapping = pd.DataFrame.from_dict(mapping)
        return mapping.set_index(org).to_dict()[dest]
    
    def variable_mapping(self,
                         org = 'native' , 
                         dest='PIK'):
    
        mapping = {
            'native' : [
                'Emissions|CO2', 'Emissions|CO2|Fossil Fuels and Industry', 
                'Emissions|CO2|Land Use', 'Emissions|Kyoto Gases',  
                'Emissions|CH4', 'Emissions|F-Gases', 'Emissions|N2O', 
                'Emissions|BC', 'Emissions|CO', 'Emissions|Sulfur', 'Emissions|NOx', 
                'Emissions|OC'
                'Population','GDP|MER', 'Policy Cost|Area under MAC Curve', 
                'Policy Cost|Additional Total Energy System Cost', 'Price|Carbon', 
                'Consumption', 'Policy Cost| Consumption Loss', 'Policy Cost|GDP Loss', 
                'Policy Cost|Other'],
            'PIK': [
                'CO2CAT0', 'CO2CATM0EL','CO2CAT5','KYOTOGHGCAT0','CH4CAT0','FGASESCAT0',
                'N2OCAT0CAT0', 'BCCAT0', 'COCAT0', 'SOXCAT0', 'NOXOCCAT0', 'POPDEMOGR',
                'GDPECO', 'AUMACCECO', 'ATESCOSTECO', 'CPRICEECO', 'CONSECO', 'CONSLOSSECO',
                'GDPLOSSECO','OCOSTSECO']}
        mapping = pd.DataFrame.from_dict(mapping)
        
        return mapping.set_index(org).to_dict()[dest]
        
    def scenario_mapping(self,
                         org = 'native', 
                         dest='PIK'):
        
        mapping = {'native' : ['EMF27-450-Conv', 'EMF27-450-EERE', 'EMF27-450-FullTech',
       'EMF27-450-LimBio', 'EMF27-450-LimSW', 'EMF27-450-LowEI',
       'EMF27-450-NoCCS', 'EMF27-450-NucOff', 'EMF27-550-Conv',
       'EMF27-550-EERE', 'EMF27-550-FullTech', 'EMF27-550-LimBio',
       'EMF27-550-LimSW', 'EMF27-550-LimTech', 'EMF27-550-LowEI',
       'EMF27-550-NoCCS', 'EMF27-550-NucOff', 'EMF27-Base-Conv',
       'EMF27-Base-EERE', 'EMF27-Base-FullTech', 'EMF27-Base-LimBio',
       'EMF27-Base-LimSW', 'EMF27-Base-LimTech', 'EMF27-Base-LowEI',
       'EMF27-Base-NucOff', 'EMF27-FP-EERE', 'EMF27-FP-FullTech',
       'EMF27-G8-EERE', 'EMF27-G8-FullTech', 'LIMITS-450', 'LIMITS-500',
       'LIMITS-Base', 'LIMITS-RefPol', 'LIMITS-RefPol-450',
       'LIMITS-RefPol-450-EE', 'LIMITS-RefPol-450-PC',
       'LIMITS-RefPol-500', 'LIMITS-RefPol2030-500', 'LIMITS-StrPol',
       'LIMITS-StrPol-450', 'LIMITS-StrPol-500', 'AME 2.6 W/m2 OS',
       'AME 3.7 W/m2 NTE', 'AME CO2 price $10 (5% p.a.)',
       'AME CO2 price $30 (5% p.a.)', 'AME CO2 price $50 (5% p.a.)',
       'AME Reference', 'AMPERE2-450-FullTech-HST',
       'AMPERE2-450-FullTech-LST', 'AMPERE2-450-FullTech-OPT',
       'AMPERE2-450-LowEI-HST', 'AMPERE2-450-LowEI-LST',
       'AMPERE2-450-LowEI-OPT', 'AMPERE2-450-NoCCS-HST',
       'AMPERE2-450-NoCCS-LST', 'AMPERE2-450-NoCCS-OPT',
       'AMPERE2-450-NucOff-HST', 'AMPERE2-450-NucOff-LST',
       'AMPERE2-450-NucOff-OPT', 'AMPERE2-550-FullTech-HST',
       'AMPERE2-550-FullTech-LST', 'AMPERE2-550-FullTech-OPT',
       'AMPERE2-Base-FullTech-OPT', 'AMPERE2-Base-LowEI-OPT',
       'AMPERE2-Base-NucOff-OPT', 'AMPERE3-450', 'AMPERE3-450P-EU',
       'AMPERE3-550', 'AMPERE3-Base', 'AMPERE3-CF450', 'AMPERE3-CF550',
       'AMPERE3-RefP-EUback', 'AMPERE3-RefPol', 'Carbon tax 0',
       'Carbon tax 100', 'Carbon tax 150', 'Carbon tax 200',
       'Carbon tax 250', 'Carbon tax 300', 'Carbon tax 350',
       'Carbon tax 400', 'Carbon tax 450', 'Carbon tax 50',
       'Carbon tax 500', 'TER2011', 'AMPERE2-450-Conv-HST',
       'AMPERE2-450-Conv-LST', 'AMPERE2-450-Conv-OPT',
       'AMPERE2-450-EERE-HST', 'AMPERE2-450-EERE-LST',
       'AMPERE2-450-EERE-OPT', 'AMPERE2-450-LimBio-HST',
       'AMPERE2-450-LimBio-LST', 'AMPERE2-450-LimBio-OPT',
       'AMPERE2-450-LimSW-HST', 'AMPERE2-450-LimSW-LST',
       'AMPERE2-450-LimSW-OPT', 'AMPERE2-550-Conv-OPT',
       'AMPERE2-550-EERE-OPT', 'AMPERE2-550-LimBio-OPT',
       'AMPERE2-550-LimSW-OPT', 'AMPERE2-550-LowEI-OPT',
       'AMPERE2-550-NoCCS-OPT', 'AMPERE2-550-NucOff-OPT',
       'AMPERE2-Base-Conv-OPT', 'AMPERE2-Base-EERE-OPT',
       'AMPERE2-Base-LimBio-OPT', 'AMPERE2-Base-LimSW-OPT',
       'AMPERE3-450P-CE', 'AMPERE3-Base-EUback', 'AMPERE3-CF450P-EU',
       'AMPERE3-RefP-CEback', 'ROSE 450 DEF', 'ROSE 450 FS Gr',
       'ROSE 450 HI Coal', 'ROSE 450 HI Fos', 'ROSE 450 HI Gas',
       'ROSE 450 HI Pop', 'ROSE 450 LO Fos', 'ROSE 450 LO Oil',
       'ROSE 450 SL Gr', 'ROSE 550 DEF', 'ROSE 550 FS Gr',
       'ROSE 550 HI Coal', 'ROSE 550 HI Fos', 'ROSE 550 HI Gas',
       'ROSE 550 HI Pop', 'ROSE 550 LO Fos', 'ROSE 550 LO Oil',
       'ROSE 550 SL Gr', 'ROSE BAU DEF', 'ROSE BAU FS Gr',
       'ROSE BAU HI Coal', 'ROSE BAU HI Fos', 'ROSE BAU HI Gas',
       'ROSE BAU HI Pop', 'ROSE BAU LO Fos', 'ROSE BAU LO Oil',
       'ROSE BAU SL Gr', 'ROSE WEAK-2020 DEF', 'ROSE WEAK-2030 DEF',
       'ROSE WEAK-POL DEF', 'Advanced Technology Scenario',
       'Reference Scenario', 'Level1', 'Level2', 'Level3', 'Level4',
       'REF', 'AMPERE3-550P-EU', 'POEM_ScenarioA', 'POEM_ScenarioA_CPH',
       'POEM_ScenarioB_CPH_early', 'POEM_baseline', 'EMF22 2.6 OS BECCS',
       'EMF22 3.7 NTE', 'EMF22 3.7 NTE w Delay', 'EMF22 3.7 OS',
       'EMF22 3.7 OS w Delay', 'EMF22 4.5 NTE', 'EMF22 4.5 NTE w Delay',
       'EMF22 Reference', 'AMPERE3-Base-EU', 'AMPERE3-RefP-CE',
       'AMPERE3-RefP-EU', 'EMF22 3.7 NTE Pessimistic Growth',
       'EMF22 3.7 NTE w Delay Pessimistic Growth',
       'EMF22 3.7 OS Pessimistic Growth',
       'EMF22 3.7 OS w Delay Pessimistic Growth',
       'EMF22 4.5 NTE Pessimistic Growth',
       'EMF22 4.5 NTE w Delay Pessimistic Growth',
       'EMF22 Reference Pessimistic Growth', 'EMF22 2.6 OS',
       'EMF22 2.6 OS No BECCS', 'EMF22 3.7 NTE No BECCS',
       'EMF22 3.7 OS No BECCS', 'EMF22 4.5 NTE No BECCS', 'RCP 8.5',
       'RCP 8.5_MIT_2.6W', 'RCP 8.5_MIT_4.5W', 'RCP 8.5_MIT_6W',
       'GEA Counterfactual', 'GEA Efficiency_450_Illustrative',
       'GEA Efficiency_450_adv.transp_full',
       'GEA Efficiency_450_adv.transp_limbe',
       'GEA Efficiency_450_adv.transp_limbe_limir',
       'GEA Efficiency_450_adv.transp_limir',
       'GEA Efficiency_450_adv.transp_nbecs',
       'GEA Efficiency_450_adv.transp_nbecs_nsink_limbe',
       'GEA Efficiency_450_adv.transp_noccs',
       'GEA Efficiency_450_adv.transp_noccs_nonuc',
       'GEA Efficiency_450_adv.transp_nsink',
       'GEA Efficiency_450_conv.transp_full',
       'GEA Efficiency_450_conv.transp_limbe',
       'GEA Efficiency_450_conv.transp_limbe_limir',
       'GEA Efficiency_450_conv.transp_limir',
       'GEA Efficiency_450_conv.transp_nbecs',
       'GEA Efficiency_450_conv.transp_nbecs_nsink_limbe',
       'GEA Efficiency_450_conv.transp_noccs',
       'GEA Efficiency_450_conv.transp_noccs_nonuc',
       'GEA Efficiency_450_conv.transp_nonuc',
       'GEA Efficiency_450_conv.transp_nsink', 'GEA Mix_450_Illustrative',
       'GEA Mix_450_adv.transp_full', 'GEA Mix_450_adv.transp_limbe',
       'GEA Mix_450_adv.transp_limir', 'GEA Mix_450_adv.transp_nbecs',
       'GEA Mix_450_adv.transp_noccs', 'GEA Mix_450_adv.transp_nonuc',
       'GEA Mix_450_adv.transp_nsink', 'GEA Mix_450_conv.transp_limbe',
       'GEA Mix_450_conv.transp_limir', 'GEA Mix_450_conv.transp_nbecs',
       'GEA Mix_450_conv.transp_nonuc', 'GEA Mix_450_conv.transp_nsink',
       'GEA Supply_450_Illustrative', 'GEA Supply_450_adv.transp_limbe',
       'GEA Supply_450_adv.transp_limir',
       'GEA Supply_450_adv.transp_nbecs',
       'GEA Supply_450_adv.transp_nonuc',
       'GEA Supply_450_adv.transp_nsink',
       'GEA Supply_450_conv.transp_full',
       'GEA Supply_450_conv.transp_nonuc', 'EMF22 2.6 NTE',
       'EMF22 2.6 OS Lo Tech', 'EMF22 2.6 OS w Delay',
       'EMF22 3.7 NTE Lo Tech', 'EMF22 3.7 OS Lo Tech',
       'EMF22 3.7 OS w Delay Lo Tech', 'EMF22 4.5 NTE Lo Tech',
       'EMF22 4.5 NTE w Delay Lo Tech', 'EMF27-450-LimTech', '400ppm',
       '400ppmbiomax', '400ppmbiomin', '400ppmccsmin', '400ppmnonuke',
       '450ppm', '550ppm', '550ppmbiomax', '550ppmbiomin', '550ppmnoccs',
       '550ppmnonuke', '550ppmnorenew', 'baseline', 'baselinebiomax',
       'baselinebiomin', '410CO2os-DEF', '450CO2-EUonly',
       '450CO2-IC+CHN+IND', '450CO2-IConly', '450CO2-delay2020',
       '450CO2-fixBIO', '450CO2-fixNUC', '450CO2-fixRET', '450CO2-noCCS',
       '450CO2-noCCSfixNUC', '450CO2nte-DEF', '450CO2os-DEF', 'Baseline',
       'ROSE 450 FS Gr SL Con', 'ROSE 550 FS Gr SL Con',
       'ROSE BAU FS Gr SL Con', 'ROSE BAU LO Oil HI Gas',
       'ROSE BAU SL Gr SL Con', 'ROSE WEAK-2020 FS Gr',
       'ROSE WEAK-2020 FS Gr SL Con', 'ROSE WEAK-2020 SL Gr',
       'ROSE WEAK-2030 FS Gr', 'ROSE WEAK-2030 FS Gr SL Con',
       'ROSE WEAK-2030 SL Gr', 'ROSE WEAK-POL FS Gr',
       'ROSE WEAK-POL FS Gr SL Con', 'ROSE WEAK-POL SL Gr']}
        
        
        strings_to_replace={
            'Reference': 'REF',
            'AMPERE': 'AMP',
            'FullTech': 'FT',
            'price $': 'P',
            '(5% p.a.)': '',
            'LimBio': 'LB',
            'LimSW': 'LSW',
            'LowEI': 'LEI',
            'NoCCS': 'NCCS',
            'NucOff': 'NOFF',
            'baseline': 'BL',
            'Baseline': 'BL',
            'Base': 'BL',
            'back': 'B',
            'No BECCS': 'NBCS',
            'Pessimistic Growth': 'PG',
            'w Delay': 'WD',
            'Counterfactual': 'CF',
            'Illustrative': 'ILL',
            'adv.transp': 'AT',
            'conv.transp': 'CT',
            'limbe': 'LB',
            'Efficiency': 'EFF',
            'Supply': 'SUP',
            'full': 'F',
            'limir': 'LI',
            'nbecs': 'NBCS',
            'noccs': 'NCCS',
            'nonuc': 'NNUC',
            'nsink': 'NS',
            'LIMITS': 'LIM',
            'RefPol': 'RP',
            'StrPol': 'SP',
            '2020': '20',
            '2030': '30',
            'W/m2': '',
            'Rose': 'RSE',
            'WEAK': 'WK',
            'POL': 'P',
            'LO Oil': 'LO',
            'HI Coal': 'HC',
            'HI Pop': 'HP',
            'HI Fos': 'HF',
            'LO Fos': 'LF',
            'HI Gas': 'HG',
            'POEM': 'PM',
            'Scenario': 'SC'}

        characters_to_remove = ' /()-. +_'
        pattern = "[" + characters_to_remove + "]"
        PIK_scenarios = list()
        for scen in mapping['native']:
            
            for old, new in strings_to_replace.items():
                scen = scen.replace(old, new)
                
            PIK_scenarios.append(re.sub(pattern, "", scen).upper())
            
        mapping['PIK'] = PIK_scenarios
        
        mapping = pd.DataFrame.from_dict(mapping)
         
        return mapping.set_index(org).to_dict()[dest]
        
if __name__ == '__main__':

    #loc = Mapping()        
    regions = initializeRegionsFromData()
    regions.save()
    
    countries = initializeCountriesFromData()
    countries.save()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon May 25 12:32:28 2020

@author: ageiges
"""

import datatoolbox as dt
dt.admin.switch_database_to_testing()

def test_init():
    dt.inventory()
    dt.DBinfo()
    dt.sourceInfo()
    assert len(dt.sourceInfo()) == len(dt.core.DB.sources)
    
def test_DB():
    
    assert hasattr(dt.core, 'DB')
    
if __name__ == '__main__':
    test_init()
    test_DB()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Oct 27 12:23:20 2020

@author: ageiges
"""

import datatoolbox as dt
import numpy as np
import pandas as pd
import os
# import pandas.testing as npt

from util_for_testing import df

dt.admin.switch_database_to_testing()

def test_import():
    import pyam
    

    
def test_to_pyam_interface():
    #%%
    inv = dt.findp(variable = 'Numbers**')
    
    tableSet = dt.getTables(inv.index)
    
    
    idf = tableSet.to_pyam()
    
    tableSet_new = dt.interfaces.pyam.from_IamDataFrame(idf)
    
    assert tableSet.keys() == tableSet_new.keys()
    
    fields_to_check = dt.config.ID_FIELDS + dt.config.OPTIONAL_META_FIELDS
    
    keys = list(tableSet.keys())
    keys.sort()
    
    for key  in keys:
        table_in  = tableSet[key]
        table_out = tableSet_new[key]
        meta_in = [table_in.meta.get(x,'') for x in fields_to_check]
        meta_out = [table_out.meta.get(x,'') for x in fields_to_check]
        assert meta_in == meta_out
        assert (table_in.values == table_out.values).all()
        assert (table_in.index == table_out.index).all()
        assert (table_in.columns == table_out.columns).all()

def test_lossless_conversion():
    
    exp = df.copy().clean() # copy initial datatable
    
    idf = exp.to_pyam() # create IamDataFrame
    
    obs = dt.Datatable.from_pyam(idf)
    
    assert pd.testing.assert_frame_equal(obs,exp) is None

if __name__ == '__main__':
    test_import()
    test_to_pyam_interface()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jan 31 10:28:44 2019

@author: and
"""
import re
import os
import pandas as pd
from fuzzywuzzy import process
import pickle

from . import config

DATA_COLUMNS = ['formula', 
                'GWP100_AR4', 
                'GWP100_AR5', 
                'GWP100', 
                'lifetime', 
                'MAGICC6_varname',
                'MAGICC6_unit',
                'MAGICC7_varname', 
                'description','source']

ID_CONDITION_FUNC = lambda x : re.sub(r"([_-])", "", x).lower()


class GreenhouseGasTable(object):
    
    
    def __init__(self):
        
        self.data = pd.read_csv(os.path.join(config.MODULE_DATA_PATH, config.GHG_GAS_TABLE_FILE), index_col=0)
        self.idData = GasDetector()
            
            
        #self._processNameColumn()
        
#    def _processNameColumn(self):
#        stringConditioningFunc = lambda x : re.sub(r"([_-])", "", x).lower()
#        
#        self.data['condName'] = self.data['name'].apply(stringConditioningFunc)
    def __str__(self):
        return self.data.__str__()
    
    def addEntry(self, dataDict):
        id_ = dataDict.pop('id')
        index = self.findEntryIdx(id_)
        print(index)
        if index is None:
            #print(id_)
            self.data.loc[id_] = None
            self.data.loc[id_] = dataDict
            
            submitDict = {ID_CONDITION_FUNC(id_): id_}
            if 'formula' in dataDict:
                submitDict[dataDict['formula']] = id_
            print(submitDict)    
            self.idData.addGas(submitDict)
            
            return id_
        else:
            print(id_ + ' found in database')
            print('found Entry: ' + str(self.getEntry(index)))
            #print(dataDict)
        
            return index
        
    def save(self):
        self.data.to_csv(config.MODULE_DATA_PATH + config.GHG_GAS_TABLE_FILE)
        self.idData.save()
        
#    def findComponent(self, searchString):
#        
#        res, score          = process.extract(searchString, self.data.baseComponent.values, limit=1)[0]
#        #res_alt, score_alt  = process.extract(searchString).lower(), self.data.condName.values, limit=1)[0]
#        if score > 95:
#            idx = np.where(self.data.baseComponent==res)[0][0]
#            return self.data.baseComponent[idx]
#        else :
#            #return None
#        
#            mask = np.asarray([strData in searchString for strData in self.data.baseComponent])
#            
#            candidates = self.data.baseComponent[mask]
#            #print(candidates)
#            #print(len(self.data.baseComponent[mask]))
#            if len(candidates) == 0:
#                
#                return ''
#            else:
#                return(candidates.values[candidates.map(lambda x: len(x)).values.argmax()])
    def searchGHG(self, searchStr):
        id_ = self.idData.findGas(searchStr)
        if id_ is None:
            return None
        return self.getEntry(id_)
    
    def getEntry(self, id_):
        try:
            return self.data.loc[id_]
        except:
            assumedID = self.idData.findGas(id_)
            print('Inferring ' + assumedID + ' from ' + id_)
            return self.data.loc[assumedID]

    def getEntries(self, idList):
        
        ids = [self.findEntryIdx(id_) for id_ in idList]
        return self.data.loc[ids]

    def findEntryIdx(self, searchString):
        
        return self.idData.findGas(searchString)
        
        
    def updateEntry(self, index, dataDict, overwrite = False):
        """
        Adding data to the table. If overwrite is true  existing data
        will be overwritten
        """
        
        index = self.findEntryIdx(index)
        for key in dataDict.keys():
            if overwrite or pd.isna(self.data.loc[index][key]):
                self.data.loc[index][key] = dataDict[key]
            


class GasDetector(object):
    FILENAME = 'GHG_alternative_naming.pkl'
    def __init__(self, clearData = True):
        
#        if not(os.path.isfile(DATA_STORAGE_LOCATION + self.FILENAME)) or clearData:
#            print(DATA_STORAGE_LOCATION + self.FILENAME + 'not  found - init empty id data dict')
#            self.data = dict()
#        else:
        fid = open(os.path.join(config.MODULE_DATA_PATH, config.GHG_NAMING_FILENAME),'rb')
        self.data = pickle.load(fid)
        fid.close()
            
    def findGas(self, string):
        """
        Returns the main identifier from other alternative names
        """
        if len(self.data) == 0:
            return None
        
        try:
            # direct id match
            return self.data[ID_CONDITION_FUNC(string)]

        except:
            key, score = process.extract(ID_CONDITION_FUNC(string), list(self.data.keys()), limit=1)[0]

            if score > 95:
                return self.data[key]
            else :
                return None
        

    def addGas(self, newDataDict):
        
        for key in newDataDict.keys():
            self.data[ID_CONDITION_FUNC(key)] = newDataDict[key]
        
        
    def save(self):
        fid = open(config.MODULE_DATA_PATH + self.FILENAME,'wb')
        pickle.dump(self.data, fid)
        fid.close()
        
if __name__ == '__main__':

    table = GreenhouseGasTable()
    
    testData = {'id': 'Carbon Dioxide', 'formula': 'CO2', 'GWP100_AR5' : 1}
    table.addEntry(testData)
    print(table)
    testData = {'id': 'CO2', 'formula': 'CO2', 'GWP100_AR5' : 2}
    table.addEntry(testData)
    
    #adding missing names
#    eqwConf.ar4_gwp['FossilCO2'] = eqwConf.ar4_gwp['CO2']
#    eqwConf.ar4_gwp['OtherCO2'] = eqwConf.ar4_gwp['CO2']
#    eqwConf.ar4_gwp['NMVOC'] = eqwConf.ar4_gwp['VOC']
#    
#    # adding nessessary names
#    eqwConf.ar5_gwp['FossilCO2'] = eqwConf.ar5_gwp['CO2']
#    eqwConf.ar5_gwp['OtherCO2']  = eqwConf.ar5_gwp['CO2']
#    eqwConf.ar5_gwp['NMVOC']     = eqwConf.ar5_gwp['VOC']
#    eqwConf.ar5_gwp['HFC143a']   = eqwConf.ar5_gwp['HFC143A']
#    eqwConf.ar5_gwp['HFC227ea']  = eqwConf.ar5_gwp['HFC227EA']
#    eqwConf.ar5_gwp['HFC245fa']  = eqwConf.ar5_gwp['HFC245FA']
#    eqwConf.ar5_gwp['HFC134a']   = eqwConf.ar5_gwp['HFC134A']
#    eqwConf.ar5_gwp['HFC43-10']  = 1650 # IIASA HFC excel sheet
#    
#    
#    
#    for gasMeta in MAGICC6_ENTITIES:
#        ar4_gwp = eqwConf.ar4_gwp[gasMeta[1]]
#        ar5_gwp = eqwConf.ar5_gwp[gasMeta[1]]
#        table.addEntry({'name': gasMeta[0],
#                   'baseComponent': gasMeta[1],
#                   'baseUnit': gasMeta[2],
#                   'GWP100_AR4': ar4_gwp,
#                   'GWP100_AR5': ar5_gwp, 
#                   'MAGICC6': True})
#    table._processNameColumn()
#    print(table.data)
#    x = table.findComponent('FossilCO2')
#    
#    table.findEntry('HFC-143')
#    table.save()
#    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed May 19 09:49:33 2021

@author: ageiges
"""

""" 
Optional toosl for the automatic creation of word documents
"""
import os
import matplotlib.pyplot as plt
import pandas as pd

import docx
# from docx import Document
from docx.shared import Inches
from docx.enum.text import WD_ALIGN_PARAGRAPH, WD_COLOR_INDEX

import numpy as np
from PIL import ImageColor, Image
from docx.shared import RGBColor, Pt
from docx.oxml.ns import nsdecls
from docx.oxml import parse_xml

#%% Defintions
alignments = {
    'left' : WD_ALIGN_PARAGRAPH.LEFT,
    'right' : WD_ALIGN_PARAGRAPH.RIGHT,
    'center' : WD_ALIGN_PARAGRAPH.CENTER,
    'block' : WD_ALIGN_PARAGRAPH.JUSTIFY,
    'justify' : WD_ALIGN_PARAGRAPH.JUSTIFY}

def highlight_runs(para, 
                   index_to_highlight,
                   color = WD_COLOR_INDEX.YELLOW):
    
    for idx in index_to_highlight:
        para.runs[idx].font.highlight_color=color
        
def _crop_png(file):
    def bbox(im):
        a = np.array(im)[:,:,:3]  # keep RGB only
        m = np.any(a != [255, 255, 255], axis=2)
        coords = np.argwhere(m)
        y0, x0, y1, x1 = *np.min(coords, axis=0), *np.max(coords, axis=0)
        return (max(0,x0-5), max(0,y0-5), x1, y1)

    im = Image.open(file)
    print(bbox(im))  # (33, 12, 223, 80)
    im2 = im.crop(bbox(im))
    im2.save(file)
   
def set_cell_background(table, 
                        row,
                        col, 
                        color):
    
    if color.startswith('#'):
        color = color[1:]
    shading_elm_1 = parse_xml(r'<w:shd {} w:fill="{}"/>'.format(nsdecls('w'), color))
    table.rows[row].cells[col]._tc.get_or_add_tcPr().append(shading_elm_1)

    return table

def change_font_size(table,
                     size):
    for row in table.rows:
        for cell in row.cells:
            paragraphs = cell.paragraphs
            for paragraph in paragraphs:
                for run in paragraph.runs:
                    font = run.font
                    font.size= Pt(size)

def open_word_file(file_name):
    import datatoolbox as dt
    dt.util.open_file(file_name)
    

class Document():
    
    
    def __init__(self, file_name):
        self.file_name = file_name
        self.doc = docx.Document()
        # return self.doc

    def add_page_break(self,):
        self.doc.add_page_break()

    def add_heading(self, string, level):
        self.doc.add_heading(string,level=level)
      
    def add_bullet_list(self, bullet_list):
        for bullet in bullet_list:
            self.add_paragraph('•	' + bullet)
        
    def add_table(self, 
                  pandas_table, 
                  heading, 
                  float_format = '{:2.1f}',
                  ignore_index = False,
                  ignore_colums = False,
                  style = 'Light List Accent 1'):
        
        if ignore_index:
            offset = 0
        else:
            offset = 1
        self.add_paragraph('')
        n_col = len(pandas_table.columns)+offset
        table = self.doc.add_table(1, n_col)
        # table.allow_autofit = False
        # table.autofit = False
        cells = table.rows[0].cells
        a, b = cells[0], cells[-1]
        a.merge(b)
        cells[0].text = heading
        
        # populate header row --------
        if not ignore_colums:
            heading_cells = table.add_row().cells
            for i, col in enumerate(pandas_table.columns):
                 heading_cells[i+offset].text = str(col)
        # heading_cells[1].0text = 'Oil'
        # heading_cells[2].text = 'Gas'
        # heading_cells[3].text = 'Nuclear'
        # heading_cells[4].text = 'Renewables'
        for ind in pandas_table.index:
            cells = table.add_row().cells
            cells[0].text = str(ind)
            for i, col in enumerate(pandas_table.columns):
                value = pandas_table.loc[ind, col]
                if isinstance(value, float):
                    cells[i+offset].text = float_format.format(value)
                else:
                    cells[i+offset].text = str(value)
        # for cell in table.columns[0].cells:
        #     cell.width = Inches(0.5)
        # table.columns[0].width = Inches(1.0)
        # table.allow_autofit = False
        table.style = style
        return table
    
    def add_to_paragraph(self, 
                         para, 
                         text, 
                         style =None,
                         highlight_color=False):
    
        if not text.endswith(' '):
            text = text + ' '
            
    
        run =  para.add_run(text)
        if style == 'italic':
           run.italic = True
        
        elif style == 'bold':
            run.bold = True
            
        if highlight_color==True:
            run.font.highlight_color=WD_COLOR_INDEX.YELLOW
            
        return run
    
    def add_paragraph(self,
                      scentences = None,
                      alignment='left'):
        """
        
    
        Parameters
        ----------
        alignment : TYPE, optional
            DESCRIPTION. The default is 'left'.
            [left , right, center, block, justify]
    
        Returns
        -------
        None.
    
        """
        
        
        para = self.doc.add_paragraph('')
        
        if isinstance(scentences, str):
            #create iterable
            scentences = [scentences]
            
        if scentences is not None:
            for scentence in scentences:
                if not scentence.endswith(' '):
                    scentence = scentence + ' '
                self.add_to_paragraph(para, scentence)
    
            
        para.alignment = alignments[alignment]
            
        return para
    
    def add_figure(self, 
                   fig,
                   path = None, 
                   relative_width = 1., 
                   crop= False):
        
        if path is None:
            path = 'temp.png'
        
        fig.savefig('temp.png', dpi=300)
        # sdf
        
        # width = Inches(6.96)
        # height =  Inches(6.96)*ratio
        
        if crop:
            _crop_png('temp.png')
            
        im = Image.open('temp.png')
        width = Inches(5.5) * relative_width
        ratio = im.size[1]/ im.size[0]
        height = width * ratio
        
        self.doc.add_picture(path, 
                        width,
                        height, 
                        )
        
        #plt.close()
         
        os.remove(path)
        
    def highlight_runs(self, para, 
                       index_to_highlight,
                       color = WD_COLOR_INDEX.YELLOW):
        
        for idx in index_to_highlight:
            para.runs[idx].font.highlight_color=color
    
       
    def set_cell_background(table, 
                            row,
                            col, 
                            color):
        
        if color.startswith('#'):
            color = color[1:]
        shading_elm_1 = parse_xml(r'<w:shd {} w:fill="{}"/>'.format(nsdecls('w'), color))
        table.rows[row].cells[col]._tc.get_or_add_tcPr().append(shading_elm_1)
    
        return table
    
    def save(self, 
             file_name=None):
        
        if file_name is None:
            file_name = self.file_name
        self.doc.save(file_name)
    
    def open_word(self):
        self.doc.save(self.file_name)
        import datatoolbox as dt
        dt.util.open_file(self.file_name)
    

#%% Test
if __name__ == '__main__':
    data = pd.DataFrame(index = [2018, 2017], columns =  ['Coal', 'Oil','Gas','Nuclear','Renewables'])
    data.loc[2018,: ] = [88.9,0.1, 0, 4.5, 6.6]
    data.loc[2017,: ] = [89.9,0.1, 0, 3.5, 6.6]
    #data.loc[2016,: ] = [85.9,0.1, 0, 3.5, 9.6]
        
    
    doc = create_document()
    
    add_heading(doc, 'Test chapter', level=1)
    add_paragraph(doc, 'This document is generated automatically by datatoolbox using the python package docx.')
    table = add_table(data, 'Power generation shares', doc)
    
    fig =plt.figure()
    plt.clf()
    plt.bar(list(range(len(data.columns))),data.loc[2018,:])
    plt.xticks(list(range(len(data.columns))), data.columns)
    
    add_figure(doc, fig, relative_width=.7,crop=True)
    doc.save('test.docx')
    
    open_word_file('test.docx')


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr 30 19:32:13 2020

@author: Andreas Geiges

SOURCES Tutorial
"""

import datatoolbox as dt
import os

#%% Each source has its own folders  (see datashelf)
path_to_data = dt.config.PATH_TO_DATASHELF
print(path_to_data)

# containing the raw_data
print(os.listdir(os.path.join(path_to_data, 'rawdata')))


# containing the processed data as csv files (e.g. for the WDI 2018)
for file in (os.listdir(os.path.join(path_to_data, 'database/WDI_2019'))):
    print(file)

#%% all sources (should) also be organized in dt.sources

print(dt.sources)

# each containing the raw_data_read_in_class (see rawSources.py)
type(dt.sources.PRIMAP_2019)

primapReader = dt.sources.PRIMAP_2019

# the reader allows you to access the rawdata (windows not implemented, sorry)
primapReader.openRawData()

# the reader allows you to access the maping file  (windows not implemented, sorry)
primapReader.openMappingFile()

# The mapping files is doing all naming conversions as well as unit handling

#%% New data would be read in using the following steps:

# create reader instance e.g. the primap reader
primapReader = dt.sources.PRIMAP_2019

# gather data and review it
tableList, excludedTables = primapReader.gatherMappedData()

# commit tables to the dataself
dt.commitTables(tableList, 'PRIMAP 2019 update', primapReader.meta)

# same works with the other sources